@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -50,6 +50,7 @@
 #include <curl/easy.h>
 #include <glib.h>
 #include <gio/gio.h>
+#include <glib/gstdio.h>
 
 #ifdef HAVE_NCURSESW_NCURSES_H
 #include <ncursesw/ncurses.h>
@@ -61,6 +62,7 @@
 
 #include "log.h"
 #include "common.h"
+#include "config/files.h"
 
 #ifdef HAVE_GIT_VERSION
 #include "gitversion.h"
@@ -73,6 +75,7 @@ struct curl_data_t
 };
 
 static size_t _data_callback(void* ptr, size_t size, size_t nmemb, void* data);
+static gchar* _get_file_or_linked(gchar* loc);
 
 /**
  * Frees the memory allocated for a gchar* string.
@@ -127,6 +130,104 @@ auto_free_char(char** str)
     free(*str);
 }
 
+/**
+ * Closes the file descriptor.
+ *
+ * @param fd file descriptor handle. If NULL, no action is taken.
+ */
+void
+auto_close_gfd(gint* fd)
+{
+    if (fd == NULL)
+        return;
+
+    if (close(*fd) == EOF)
+        log_error(g_strerror(errno));
+}
+
+/**
+ * Closes file stream
+ *
+ * @param fd file descriptor handle opened with fopen. If NULL, no action is taken.
+ */
+void
+auto_close_FILE(FILE** fd)
+{
+    if (fd == NULL)
+        return;
+
+    if (fclose(*fd) == EOF)
+        log_error(g_strerror(errno));
+}
+
+static gboolean
+_load_keyfile(prof_keyfile_t* keyfile)
+{
+    GError* error = NULL;
+    keyfile->keyfile = g_key_file_new();
+
+    if (g_key_file_load_from_file(keyfile->keyfile, keyfile->filename, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, &error)) {
+        return TRUE;
+    } else if (error->code != G_FILE_ERROR_NOENT) {
+        log_warning("[Keyfile] error loading %s: %s", keyfile->filename, error->message);
+        g_error_free(error);
+    } else {
+        log_warning("[Keyfile] no such file: %s", keyfile->filename);
+        g_error_free(error);
+    }
+    return FALSE;
+}
+
+gboolean
+load_data_keyfile(prof_keyfile_t* keyfile, const char* filename)
+{
+    auto_gchar gchar* loc = files_get_data_path(filename);
+    return load_custom_keyfile(keyfile, _get_file_or_linked(loc));
+}
+
+gboolean
+load_config_keyfile(prof_keyfile_t* keyfile, const char* filename)
+{
+    auto_gchar gchar* loc = files_get_config_path(filename);
+    return load_custom_keyfile(keyfile, _get_file_or_linked(loc));
+}
+
+gboolean
+load_custom_keyfile(prof_keyfile_t* keyfile, gchar* filename)
+{
+    keyfile->filename = filename;
+
+    if (g_file_test(keyfile->filename, G_FILE_TEST_EXISTS)) {
+        g_chmod(keyfile->filename, S_IRUSR | S_IWUSR);
+    }
+
+    return _load_keyfile(keyfile);
+}
+
+gboolean
+save_keyfile(prof_keyfile_t* keyfile)
+{
+    GError* error = NULL;
+    if (!g_key_file_save_to_file(keyfile->keyfile, keyfile->filename, &error)) {
+        log_error("[Keyfile]: saving file %s failed! %s", keyfile->filename, error->message);
+        g_error_free(error);
+        return FALSE;
+    }
+    g_chmod(keyfile->filename, S_IRUSR | S_IWUSR);
+    return TRUE;
+}
+
+void
+free_keyfile(prof_keyfile_t* keyfile)
+{
+    log_debug("[Keyfile]: free %s", STR_MAYBE_NULL(keyfile->filename));
+    if (keyfile->keyfile)
+        g_key_file_free(keyfile->keyfile);
+    keyfile->keyfile = NULL;
+    g_free(keyfile->filename);
+    keyfile->filename = NULL;
+}
+
 gboolean
 string_to_verbosity(const char* cmd, int* verbosity, gchar** err_msg)
 {
@@ -174,9 +275,12 @@ char*
 str_replace(const char* string, const char* substr,
             const char* replacement)
 {
-    char* tok = NULL;
+    const char* head = NULL;
+    const char* tok = NULL;
     char* newstr = NULL;
-    char* head = NULL;
+    char* wr = NULL;
+    size_t num_substr = 0;
+    size_t len_string, len_substr, len_replacement, len_string_remains, len_newstr;
 
     if (string == NULL)
         return NULL;
@@ -184,26 +288,40 @@ str_replace(const char* string, const char* substr,
     if (substr == NULL || replacement == NULL || (strcmp(substr, "") == 0))
         return strdup(string);
 
-    newstr = strdup(string);
-    head = newstr;
+    tok = string;
+    while ((tok = strstr(tok, substr))) {
+        num_substr++;
+        tok++;
+    }
+
+    if (num_substr == 0)
+        return strdup(string);
+
+    len_string = strlen(string);
+    len_substr = strlen(substr);
+    len_replacement = strlen(replacement);
+    len_newstr = len_string - (num_substr * len_substr) + (num_substr * len_replacement);
+    newstr = malloc(len_newstr + 1);
+    if (newstr == NULL) {
+        return NULL;
+    }
+    len_string_remains = len_string;
+
+    head = string;
+    wr = newstr;
 
     while ((tok = strstr(head, substr))) {
-        auto_char char* oldstr = newstr;
-        newstr = malloc(strlen(oldstr) - strlen(substr) + strlen(replacement) + 1);
+        size_t l = tok - head;
+        memcpy(wr, head, l);
+        wr += l;
+        memcpy(wr, replacement, len_replacement);
+        wr += len_replacement;
+        len_string_remains -= len_substr + l;
 
-        if (newstr == NULL) {
-            return NULL;
-        }
-
-        memcpy(newstr, oldstr, tok - oldstr);
-        memcpy(newstr + (tok - oldstr), replacement, strlen(replacement));
-        memcpy(newstr + (tok - oldstr) + strlen(replacement),
-               tok + strlen(substr),
-               strlen(oldstr) - strlen(substr) - (tok - oldstr));
-        memset(newstr + strlen(oldstr) - strlen(substr) + strlen(replacement), 0, 1);
-
-        head = newstr + (tok - oldstr) + strlen(replacement);
+        head = tok + len_substr;
     }
+    memcpy(wr, head, len_string_remains);
+    newstr[len_newstr] = '\0';
 
     return newstr;
 }
@@ -325,24 +443,24 @@ _data_callback(void* ptr, size_t size, size_t nmemb, void* data)
     return realsize;
 }
 
-gchar*
-get_file_or_linked(gchar* loc, gchar* basedir)
+static gchar*
+_get_file_or_linked(gchar* loc)
 {
-    gchar* true_loc = NULL;
+    auto_gchar gchar* basedir = g_path_get_dirname(loc);
+    gchar* true_loc = g_strdup(loc);
 
     // check for symlink
-    if (g_file_test(loc, G_FILE_TEST_IS_SYMLINK)) {
-        true_loc = g_file_read_link(loc, NULL);
+    while (g_file_test(true_loc, G_FILE_TEST_IS_SYMLINK)) {
+        gchar* tmp = g_file_read_link(true_loc, NULL);
+        g_free(true_loc);
+        true_loc = tmp;
 
         // if relative, add basedir
         if (!g_str_has_prefix(true_loc, "/") && !g_str_has_prefix(true_loc, "~")) {
-            gchar* tmp = g_strdup_printf("%s/%s", basedir, true_loc);
+            tmp = g_strdup_printf("%s/%s", basedir, true_loc);
             g_free(true_loc);
             true_loc = tmp;
         }
-        // use given location
-    } else {
-        true_loc = g_strdup(loc);
     }
 
     return true_loc;
@@ -564,7 +682,7 @@ format_call_external_argv(const char* template, const char* url, const char* fil
     return argv;
 }
 
-gchar*
+static gchar*
 _unique_filename(const char* filename)
 {
     gchar* unique = g_strdup(filename);
@@ -588,7 +706,7 @@ _unique_filename(const char* filename)
     return unique;
 }
 
-bool
+static bool
 _has_directory_suffix(const char* path)
 {
     return (g_str_has_suffix(path, ".")
@@ -686,13 +804,6 @@ unique_filename_from_url(const char* url, const char* path)
     return unique_filename;
 }
 
-void
-glib_hash_table_free(GHashTable* hash_table)
-{
-    g_hash_table_remove_all(hash_table);
-    g_hash_table_unref(hash_table);
-}
-
 /* build profanity version string.
  * example: 0.13.1dev.master.69d8c1f9
  */
