@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -53,7 +53,7 @@
 #include "xmpp/xmpp.h"
 #include "xmpp/jid.h"
 
-static gchar* accounts_loc;
+static prof_keyfile_t accounts_prof_keyfile;
 static GKeyFile* accounts;
 
 static Autocomplete all_ac;
@@ -61,20 +61,26 @@ static Autocomplete enabled_ac;
 
 static void _save_accounts(void);
 
+static void
+_accounts_close(void)
+{
+    autocomplete_free(all_ac);
+    autocomplete_free(enabled_ac);
+    free_keyfile(&accounts_prof_keyfile);
+    accounts = NULL;
+}
+
 void
 accounts_load(void)
 {
     log_info("Loading accounts");
+
+    prof_add_shutdown_routine(_accounts_close);
+
     all_ac = autocomplete_new();
     enabled_ac = autocomplete_new();
-    accounts_loc = files_get_data_path(FILE_ACCOUNTS);
-
-    if (g_file_test(accounts_loc, G_FILE_TEST_EXISTS)) {
-        g_chmod(accounts_loc, S_IRUSR | S_IWUSR);
-    }
-
-    accounts = g_key_file_new();
-    g_key_file_load_from_file(accounts, accounts_loc, G_KEY_FILE_KEEP_COMMENTS, NULL);
+    load_data_keyfile(&accounts_prof_keyfile, FILE_ACCOUNTS);
+    accounts = accounts_prof_keyfile.keyfile;
 
     // create the logins searchable list for autocompletion
     gsize naccounts;
@@ -88,14 +94,6 @@ accounts_load(void)
     }
 }
 
-void
-accounts_close(void)
-{
-    autocomplete_free(all_ac);
-    autocomplete_free(enabled_ac);
-    g_key_file_free(accounts);
-}
-
 char*
 accounts_find_enabled(const char* const prefix, gboolean previous, void* context)
 {
@@ -228,7 +226,7 @@ accounts_get_account(const char* const name)
         } else {
             jabber_conn_status_t conn_status = connection_get_status();
             if (conn_status == JABBER_CONNECTED) {
-                char* conf_jid = connection_jid_for_feature(XMPP_FEATURE_MUC);
+                const char* conf_jid = connection_jid_for_feature(XMPP_FEATURE_MUC);
                 if (conf_jid) {
                     muc_service = strdup(conf_jid);
                 }
@@ -486,7 +484,7 @@ static void
 _accounts_set_string_option(const char* account_name, const char* const option, const char* const value)
 {
     if (accounts_account_exists(account_name)) {
-        g_key_file_set_string(accounts, account_name, option, value);
+        g_key_file_set_string(accounts, account_name, option, value ?: "");
         _save_accounts();
     }
 }
@@ -959,11 +957,5 @@ accounts_get_login_status(const char* const account_name)
 static void
 _save_accounts(void)
 {
-    gsize g_data_size;
-    auto_gchar gchar* g_accounts_data = g_key_file_to_data(accounts, &g_data_size, NULL);
-
-    auto_gchar gchar* base = g_path_get_dirname(accounts_loc);
-    auto_gchar gchar* true_loc = get_file_or_linked(accounts_loc, base);
-    g_file_set_contents(true_loc, g_accounts_data, g_data_size, NULL);
-    g_chmod(accounts_loc, S_IRUSR | S_IWUSR);
+    save_keyfile(&accounts_prof_keyfile);
 }
