@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -69,7 +69,7 @@
 
 #define INPBLOCK_DEFAULT 1000
 
-static gchar* prefs_loc;
+static prof_keyfile_t prefs_prof_keyfile;
 static GKeyFile* prefs;
 gint log_maxsize = 0;
 
@@ -144,16 +144,6 @@ _prefs_load(void)
         g_key_file_remove_key(prefs, PREF_GROUP_UI, "titlebar.goodbye", NULL);
     }
 
-    // after 0.8.1: titlebar use jid|name -> titlebar show|hide jid|name
-    if (g_key_file_has_key(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL)) {
-        auto_gchar gchar* value = g_key_file_get_string(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL);
-        if (g_strcmp0(value, "name") == 0) {
-            g_key_file_set_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", TRUE);
-        } else if (g_strcmp0(value, "jid") == 0) {
-            g_key_file_set_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.jid", TRUE);
-        }
-    }
-
     // 0.9.0 introduced /urlopen. It was saved under "logging" section. Now we have a new "executables" section.
     if (g_key_file_has_key(prefs, PREF_GROUP_LOGGING, "urlopen.cmd", NULL)) {
         auto_gchar gchar* val = g_key_file_get_string(prefs, PREF_GROUP_LOGGING, "urlopen.cmd", NULL);
@@ -208,6 +198,50 @@ _prefs_load(void)
         g_key_file_remove_key(prefs, PREF_GROUP_PLUGINS, "sourcepath", NULL);
     }
 
+    // after 0.14.0: "/roster room use name|jid" and "/roster room show|hide server"
+    // became "/roster room title bookmark|jid|localpart|name".
+    // The outer-most if-statement is not needed, but it is included for better performance.
+    if (g_key_file_has_key(prefs, PREF_GROUP_UI, "roster.rooms.use.name", NULL) || g_key_file_has_key(prefs, PREF_GROUP_UI, "roster.rooms.server", NULL)) {
+        GError* err = NULL;
+        auto_gchar gchar* pref_use_name = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.rooms.use.name", NULL);
+        if (g_strcmp0(pref_use_name, "jid") == 0) {
+            const gboolean pref_show_server = g_key_file_get_boolean(prefs, PREF_GROUP_UI, "roster.rooms.server", &err);
+            if (!pref_show_server && !err) {
+                g_key_file_set_string(prefs, PREF_GROUP_UI, "roster.rooms.title", "localpart");
+            } else {
+                g_key_file_set_string(prefs, PREF_GROUP_UI, "roster.rooms.title", "jid");
+            }
+        }
+        g_clear_error(&err);
+        g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.rooms.server", NULL);
+        g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.rooms.use.name", NULL);
+    }
+
+    // after 0.14.0: "/statusbar room room|jid" became "/statusbar room title bookmark|jid|localpart|name"
+    if (g_key_file_has_key(prefs, PREF_GROUP_UI, "statusbar.room", NULL)) {
+        auto_gchar gchar* statusbar_room = g_key_file_get_string(prefs, PREF_GROUP_UI, "statusbar.room", NULL);
+        if (g_strcmp0(statusbar_room, "room") == 0) {
+            g_key_file_set_string(prefs, PREF_GROUP_UI, "statusbar.room.title", "localpart");
+        } else if (g_strcmp0(statusbar_room, "jid") == 0) {
+            g_key_file_set_string(prefs, PREF_GROUP_UI, "statusbar.room.title", "jid");
+        }
+        g_key_file_remove_key(prefs, PREF_GROUP_UI, "statusbar.room", NULL);
+    }
+
+    // after 0.14.0: "/statusbar show jid" and "/statusbar show name"
+    // became "/titlebar room title bookmark|jid|localpart|name"
+    // The outer-most if-statement is not needed, but it is included for better performance.
+    if (g_key_file_has_key(prefs, PREF_GROUP_UI, "titlebar.muc.title.jid", NULL) || g_key_file_has_key(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", NULL)) {
+        GError* err = NULL;
+        const gboolean pref_titlebar_muc_title_jid = g_key_file_get_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.jid", NULL);
+        const gboolean pref_titlebar_muc_title_name = g_key_file_get_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", &err);
+        if (pref_titlebar_muc_title_jid && !pref_titlebar_muc_title_name && !err) {
+            g_key_file_set_string(prefs, PREF_GROUP_UI, "titlebar.muc.title", "jid");
+        }
+        g_clear_error(&err);
+        g_key_file_remove_key(prefs, PREF_GROUP_UI, "titlebar.muc.title.jid", NULL);
+        g_key_file_remove_key(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", NULL);
+    }
     _save_prefs();
 
     boolean_choice_ac = autocomplete_new();
@@ -234,37 +268,20 @@ _prefs_close(void)
 void
 prefs_reload(void)
 {
-    /*
-     * Current function contains copy-paste, but we wanted to avoid config_file
-     * manipulation from prefs_load/prefs_close
-     */
-
-    _prefs_close();
-
-    g_key_file_free(prefs);
-    prefs = NULL;
-
-    prefs = g_key_file_new();
-    g_key_file_load_from_file(prefs, prefs_loc, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-    _prefs_load();
+    auto_gchar gchar* loc = g_strdup(prefs_prof_keyfile.filename);
+    prefs_close();
+    prefs_load(loc);
 }
 
 void
-prefs_load(char* config_file)
+prefs_load(const char* config_file)
 {
     if (config_file == NULL) {
-        prefs_loc = files_get_config_path(FILE_PROFRC);
+        load_config_keyfile(&prefs_prof_keyfile, FILE_PROFRC);
     } else {
-        prefs_loc = g_strdup(config_file);
+        load_custom_keyfile(&prefs_prof_keyfile, g_strdup(config_file));
     }
-
-    if (g_file_test(prefs_loc, G_FILE_TEST_EXISTS)) {
-        g_chmod(prefs_loc, S_IRUSR | S_IWUSR);
-    }
-
-    prefs = g_key_file_new();
-    g_key_file_load_from_file(prefs, prefs_loc, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
+    prefs = prefs_prof_keyfile.keyfile;
 
     _prefs_load();
 }
@@ -280,11 +297,8 @@ prefs_close(void)
 {
     _prefs_close();
 
-    g_key_file_free(prefs);
+    free_keyfile(&prefs_prof_keyfile);
     prefs = NULL;
-
-    g_free(prefs_loc);
-    prefs_loc = NULL;
 }
 
 gchar*
@@ -900,11 +914,11 @@ prefs_get_occupants_size(void)
     }
 }
 
-char*
+gchar*
 prefs_get_occupants_char(void)
 {
 
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.char", NULL);
+    gchar* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.char", NULL);
 
     return result;
 }
@@ -946,10 +960,10 @@ prefs_set_occupants_indent(gint value)
     g_key_file_set_integer(prefs, PREF_GROUP_UI, "occupants.indent", value);
 }
 
-char*
+gchar*
 prefs_get_occupants_header_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.header.char", NULL);
+    gchar* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.header.char", NULL);
 
     return result;
 }
@@ -988,14 +1002,14 @@ prefs_get_roster_size(void)
     }
 }
 
-static char*
+static gchar*
 _prefs_get_encryption_char(const char* const ch, const char* const pref_group, const char* const key)
 {
-    char* result = NULL;
+    gchar* result = NULL;
 
-    char* resultstr = g_key_file_get_string(prefs, pref_group, key, NULL);
+    gchar* resultstr = g_key_file_get_string(prefs, pref_group, key, NULL);
     if (!resultstr) {
-        result = strdup(ch);
+        result = g_strdup(ch);
     } else {
         result = resultstr;
     }
@@ -1015,7 +1029,7 @@ _prefs_set_encryption_char(const char* const ch, const char* const pref_group, c
     return FALSE;
 }
 
-char*
+gchar*
 prefs_get_otr_char(void)
 {
     return _prefs_get_encryption_char("~", PREF_GROUP_OTR, "otr.char");
@@ -1027,7 +1041,7 @@ prefs_set_otr_char(char* ch)
     return _prefs_set_encryption_char(ch, PREF_GROUP_OTR, "otr.char");
 }
 
-char*
+gchar*
 prefs_get_pgp_char(void)
 {
     return _prefs_get_encryption_char("~", PREF_GROUP_PGP, "pgp.char");
@@ -1039,7 +1053,7 @@ prefs_set_pgp_char(char* ch)
     return _prefs_set_encryption_char(ch, PREF_GROUP_PGP, "pgp.char");
 }
 
-char*
+gchar*
 prefs_get_ox_char(void)
 {
     return _prefs_get_encryption_char("%", PREF_GROUP_OX, "ox.char");
@@ -1051,7 +1065,7 @@ prefs_set_ox_char(char* ch)
     return _prefs_set_encryption_char(ch, PREF_GROUP_OX, "ox.char");
 }
 
-char*
+gchar*
 prefs_get_omemo_char(void)
 {
     return _prefs_get_encryption_char("~", PREF_GROUP_OMEMO, "omemo.char");
@@ -1063,17 +1077,10 @@ prefs_set_omemo_char(char* ch)
     return _prefs_set_encryption_char(ch, PREF_GROUP_OMEMO, "omemo.char");
 }
 
-char*
+gchar*
 prefs_get_roster_header_char(void)
 {
-    char* result = NULL;
-
-    char* resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.header.char", NULL);
-    if (resultstr) {
-        result = resultstr;
-    }
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.header.char", NULL);
 }
 
 void
@@ -1092,12 +1099,10 @@ prefs_clear_roster_header_char(void)
     g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.header.char", NULL);
 }
 
-char*
+gchar*
 prefs_get_roster_contact_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.contact.char", NULL);
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.contact.char", NULL);
 }
 
 void
@@ -1116,12 +1121,10 @@ prefs_clear_roster_contact_char(void)
     g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.contact.char", NULL);
 }
 
-char*
+gchar*
 prefs_get_roster_resource_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.resource.char", NULL);
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.resource.char", NULL);
 }
 
 void
@@ -1140,12 +1143,10 @@ prefs_clear_roster_resource_char(void)
     g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.resource.char", NULL);
 }
 
-char*
+gchar*
 prefs_get_roster_private_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.private.char", NULL);
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.private.char", NULL);
 }
 
 void
@@ -1164,12 +1165,10 @@ prefs_clear_roster_private_char(void)
     g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.private.char", NULL);
 }
 
-char*
+gchar*
 prefs_get_roster_room_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.rooms.char", NULL);
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.rooms.char", NULL);
 }
 
 void
@@ -1188,12 +1187,10 @@ prefs_clear_roster_room_char(void)
     g_key_file_remove_key(prefs, PREF_GROUP_UI, "roster.rooms.char", NULL);
 }
 
-char*
+gchar*
 prefs_get_roster_room_private_char(void)
 {
-    char* result = g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.rooms.private.char", NULL);
-
-    return result;
+    return g_key_file_get_string(prefs, PREF_GROUP_UI, "roster.rooms.private.char", NULL);
 }
 
 void
@@ -1275,29 +1272,22 @@ prefs_set_roster_presence_indent(gint value)
     g_key_file_set_integer(prefs, PREF_GROUP_UI, "roster.presence.indent", value);
 }
 
-char*
+gchar*
 prefs_get_correction_char(void)
 {
-    char* result = "+";
+    gchar* resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "correction.char", NULL);
 
-    char* resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "correction.char", NULL);
-    if (!resultstr) {
-        result = strdup("+");
-    } else {
-        result = resultstr;
-    }
-
-    return result;
+    return resultstr ? resultstr : g_strdup("+");
 }
 
 void
-prefs_set_correction_char(char ch)
+prefs_set_correction_char(char* ch)
 {
-    char str[2];
-    str[0] = ch;
-    str[1] = '\0';
-
-    g_key_file_set_string(prefs, PREF_GROUP_UI, "correction.char", str);
+    if (g_utf8_strlen(ch, 4) == 1) {
+        g_key_file_set_string(prefs, PREF_GROUP_UI, "correction.char", ch);
+    } else {
+        log_error("Could not set correction char: %s", ch);
+    }
 }
 
 gboolean
@@ -1713,13 +1703,7 @@ prefs_free_aliases(GList* aliases)
 static void
 _save_prefs(void)
 {
-    gsize g_data_size;
-    auto_gchar gchar* g_prefs_data = g_key_file_to_data(prefs, &g_data_size, NULL);
-    auto_gchar gchar* base = g_path_get_dirname(prefs_loc);
-    auto_gchar gchar* true_loc = get_file_or_linked(prefs_loc, base);
-
-    g_file_set_contents(true_loc, g_prefs_data, g_data_size, NULL);
-    g_chmod(prefs_loc, S_IRUSR | S_IWUSR);
+    save_keyfile(&prefs_prof_keyfile);
 }
 
 // get the preference group for a specific preference
@@ -1781,8 +1765,7 @@ _get_group(preference_t pref)
     case PREF_ROSTER_ROOMS_BY:
     case PREF_ROSTER_ROOMS_ORDER:
     case PREF_ROSTER_ROOMS_UNREAD:
-    case PREF_ROSTER_ROOMS_SERVER:
-    case PREF_ROSTER_ROOMS_USE_AS_NAME:
+    case PREF_ROSTER_ROOMS_TITLE:
     case PREF_ROSTER_PRIVATE:
     case PREF_RESOURCE_TITLE:
     case PREF_RESOURCE_MESSAGE:
@@ -1801,10 +1784,9 @@ _get_group(preference_t pref)
     case PREF_STATUSBAR_SHOW_READ:
     case PREF_STATUSBAR_SELF:
     case PREF_STATUSBAR_CHAT:
-    case PREF_STATUSBAR_ROOM:
+    case PREF_STATUSBAR_ROOM_TITLE:
     case PREF_STATUSBAR_TABMODE:
-    case PREF_TITLEBAR_MUC_TITLE_JID:
-    case PREF_TITLEBAR_MUC_TITLE_NAME:
+    case PREF_TITLEBAR_MUC_TITLE:
     case PREF_SLASH_GUARD:
     case PREF_COMPOSE_EDITOR:
     case PREF_OUTGOING_STAMP:
@@ -2072,10 +2054,8 @@ _get_key(preference_t pref)
         return "roster.rooms.order";
     case PREF_ROSTER_ROOMS_UNREAD:
         return "roster.rooms.unread";
-    case PREF_ROSTER_ROOMS_SERVER:
-        return "roster.rooms.server";
-    case PREF_ROSTER_ROOMS_USE_AS_NAME:
-        return "roster.rooms.use.name";
+    case PREF_ROSTER_ROOMS_TITLE:
+        return "roster.rooms.title";
     case PREF_ROSTER_PRIVATE:
         return "roster.private";
     case PREF_RESOURCE_TITLE:
@@ -2086,10 +2066,8 @@ _get_key(preference_t pref)
         return "inpblock.dynamic";
     case PREF_ENC_WARN:
         return "enc.warn";
-    case PREF_TITLEBAR_MUC_TITLE_JID:
-        return "titlebar.muc.title.jid";
-    case PREF_TITLEBAR_MUC_TITLE_NAME:
-        return "titlebar.muc.title.name";
+    case PREF_TITLEBAR_MUC_TITLE:
+        return "titlebar.muc.title";
     case PREF_PGP_LOG:
         return "log";
     case PREF_PGP_SENDFILE:
@@ -2130,8 +2108,8 @@ _get_key(preference_t pref)
         return "statusbar.self";
     case PREF_STATUSBAR_CHAT:
         return "statusbar.chat";
-    case PREF_STATUSBAR_ROOM:
-        return "statusbar.room";
+    case PREF_STATUSBAR_ROOM_TITLE:
+        return "statusbar.room.title";
     case PREF_STATUSBAR_TABMODE:
         return "statusbar.tabmode";
     case PREF_OMEMO_LOG:
@@ -2204,7 +2182,6 @@ _get_default_boolean(preference_t pref)
     case PREF_ROSTER_CONTACTS:
     case PREF_ROSTER_UNSUBSCRIBED:
     case PREF_ROSTER_ROOMS:
-    case PREF_ROSTER_ROOMS_SERVER:
     case PREF_TLS_SHOW:
     case PREF_LASTACTIVITY:
     case PREF_TRAY_READ:
@@ -2218,7 +2195,6 @@ _get_default_boolean(preference_t pref)
     case PREF_CARBONS:
     case PREF_STATES:
     case PREF_OUTTYPE:
-    case PREF_TITLEBAR_MUC_TITLE_NAME:
     case PREF_COLOR_NICK_OWN:
     case PREF_INTYPE:
     case PREF_INTYPE_CONSOLE:
@@ -2261,7 +2237,7 @@ _get_default_string(preference_t pref)
     case PREF_STATUSES_CHAT:
     case PREF_STATUSES_MUC:
         return "none";
-    case PREF_ROSTER_ROOMS_USE_AS_NAME:
+    case PREF_ROSTER_ROOMS_TITLE:
         return "name";
     case PREF_ROSTER_ROOMS_ORDER:
         return "name";
@@ -2297,10 +2273,12 @@ _get_default_string(preference_t pref)
         return "fulljid";
     case PREF_STATUSBAR_CHAT:
         return "user";
-    case PREF_STATUSBAR_ROOM:
-        return "room";
+    case PREF_STATUSBAR_ROOM_TITLE:
+        return "name";
     case PREF_STATUSBAR_TABMODE:
         return "default";
+    case PREF_TITLEBAR_MUC_TITLE:
+        return "name";
     case PREF_OMEMO_LOG:
         return "on";
     case PREF_OMEMO_POLICY:
