@@ -51,8 +51,6 @@ typedef enum {
     PROF_OMEMOPOLICY_ALWAYS
 } prof_omemopolicy_t;
 
-typedef struct omemo_context_t omemo_context;
-
 typedef struct omemo_key
 {
     unsigned char* data;
@@ -63,7 +61,6 @@ typedef struct omemo_key
 } omemo_key_t;
 
 void omemo_init(void);
-void omemo_close(void);
 void omemo_on_connect(ProfAccount* account);
 void omemo_on_disconnect(void);
 void omemo_generate_crypto_materials(ProfAccount* account);
