@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2018 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -63,7 +63,7 @@
 static const char* libversion = NULL;
 static GHashTable* pubkeys;
 
-static gchar* pubsloc;
+static prof_keyfile_t pubkeys_prof_keyfile;
 static GKeyFile* pubkeyfile;
 
 static char* passphrase;
@@ -111,38 +111,16 @@ _p_gpg_passphrase_cb(void* hook, const char* uid_hint, const char* passphrase_in
     return 0;
 }
 
-void
-p_gpg_init(void)
-{
-    libversion = gpgme_check_version(NULL);
-    log_debug("GPG: Found gpgme version: %s", libversion);
-    gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-
-    pubkeys = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_p_gpg_free_pubkeyid);
-
-    key_ac = autocomplete_new();
-    GHashTable* keys = p_gpg_list_keys();
-    p_gpg_free_keys(keys);
-
-    passphrase = NULL;
-    passphrase_attempt = NULL;
-}
-
-void
-p_gpg_close(void)
+static void
+_p_gpg_close(void)
 {
     if (pubkeys) {
         g_hash_table_destroy(pubkeys);
         pubkeys = NULL;
     }
 
-    if (pubkeyfile) {
-        g_key_file_free(pubkeyfile);
-        pubkeyfile = NULL;
-    }
-
-    free(pubsloc);
-    pubsloc = NULL;
+    free_keyfile(&pubkeys_prof_keyfile);
+    pubkeyfile = NULL;
 
     autocomplete_free(key_ac);
     key_ac = NULL;
@@ -158,22 +136,36 @@ p_gpg_close(void)
     }
 }
 
+void
+p_gpg_init(void)
+{
+    libversion = gpgme_check_version(NULL);
+    log_debug("GPG: Found gpgme version: %s", libversion);
+    gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
+
+    prof_add_shutdown_routine(_p_gpg_close);
+
+    pubkeys = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_p_gpg_free_pubkeyid);
+
+    key_ac = autocomplete_new();
+    GHashTable* keys = p_gpg_list_keys();
+    p_gpg_free_keys(keys);
+
+    passphrase = NULL;
+    passphrase_attempt = NULL;
+}
+
 void
 p_gpg_on_connect(const char* const barejid)
 {
-    pubsloc = files_file_in_account_data_path(DIR_PGP, barejid, "pubkeys");
+    gchar* pubsloc = files_file_in_account_data_path(DIR_PGP, barejid, "pubkeys");
     if (!pubsloc) {
         log_error("Could not create directory for account %s.", barejid);
         cons_show_error("Could not create directory for account %s.", barejid);
         return;
     }
-
-    if (g_file_test(pubsloc, G_FILE_TEST_EXISTS)) {
-        g_chmod(pubsloc, S_IRUSR | S_IWUSR);
-    }
-
-    pubkeyfile = g_key_file_new();
-    g_key_file_load_from_file(pubkeyfile, pubsloc, G_KEY_FILE_KEEP_COMMENTS, NULL);
+    load_custom_keyfile(&pubkeys_prof_keyfile, pubsloc);
+    pubkeyfile = pubkeys_prof_keyfile.keyfile;
 
     // load each keyid
     gsize len = 0;
@@ -218,28 +210,9 @@ p_gpg_on_connect(const char* const barejid)
 void
 p_gpg_on_disconnect(void)
 {
-    if (pubkeys) {
-        g_hash_table_destroy(pubkeys);
-        pubkeys = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_p_gpg_free_pubkeyid);
-    }
-
-    if (pubkeyfile) {
-        g_key_file_free(pubkeyfile);
-        pubkeyfile = NULL;
-    }
-
-    free(pubsloc);
-    pubsloc = NULL;
-
-    if (passphrase) {
-        free(passphrase);
-        passphrase = NULL;
-    }
-
-    if (passphrase_attempt) {
-        free(passphrase_attempt);
-        passphrase_attempt = NULL;
-    }
+    _p_gpg_close();
+    pubkeys = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)_p_gpg_free_pubkeyid);
+    key_ac = autocomplete_new();
 }
 
 gboolean
@@ -889,7 +862,8 @@ out:
 static ProfPGPKey*
 _gpgme_key_to_ProfPGPKey(gpgme_key_t key)
 {
-    if (key == NULL) {
+    if (key == NULL || key->uids == NULL
+        || key->subkeys == NULL || key->uids->uid == NULL) {
         return NULL;
     }
 
@@ -973,8 +947,5 @@ _add_header_footer(const char* const str, const char* const header, const char*
 static void
 _save_pubkeys(void)
 {
-    gsize g_data_size;
-    auto_gchar gchar* g_pubkeys_data = g_key_file_to_data(pubkeyfile, &g_data_size, NULL);
-    g_file_set_contents(pubsloc, g_pubkeys_data, g_data_size, NULL);
-    g_chmod(pubsloc, S_IRUSR | S_IWUSR);
+    save_keyfile(&pubkeys_prof_keyfile);
 }
