@@ -69,12 +69,6 @@ disable_python_threads()
     PyEval_RestoreThread(thread_state);
 }
 
-static void
-_unref_module(PyObject* module)
-{
-    Py_XDECREF(module);
-}
-
 const char*
 python_get_version_string(void)
 {
@@ -95,21 +89,19 @@ python_get_version_number(void)
 void
 python_env_init(void)
 {
-    loaded_modules = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)_unref_module);
+    loaded_modules = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)Py_XDECREF);
 
     python_init_prof();
 
     auto_gchar gchar* plugins_dir = files_get_data_path(DIR_PLUGINS);
-    GString* path = g_string_new("import sys\n");
-    g_string_append(path, "sys.path.append(\"");
-    g_string_append(path, plugins_dir);
-    g_string_append(path, "/\")\n");
+    auto_gchar gchar* path = g_strdup_printf(
+        "import sys\n"
+        "sys.path.append(\"%s/\")\n",
+        plugins_dir);
 
-    PyRun_SimpleString(path->str);
+    PyRun_SimpleString(path);
     python_check_error();
 
-    g_string_free(path, TRUE);
-
     allow_python_threads();
 }
 
@@ -318,6 +310,11 @@ python_pre_chat_message_display_hook(ProfPlugin* plugin, const char* const barej
 {
     disable_python_threads();
     PyObject* p_args = Py_BuildValue("sss", barejid, resource, message);
+    if (!p_args) {
+        log_warning("Unable to convert strings in `python_pre_chat_message_display_hook` to 'UTF8'. barejid: %s, resource: %s, message: %s", barejid, resource, message);
+        allow_python_threads();
+        return NULL;
+    }
     PyObject* p_function;
 
     PyObject* p_module = plugin->module;
@@ -408,6 +405,12 @@ python_pre_room_message_display_hook(ProfPlugin* plugin, const char* const barej
 {
     disable_python_threads();
     PyObject* p_args = Py_BuildValue("sss", barejid, nick, message);
+    if (!p_args) {
+        log_warning("Unable to convert strings in `python_pre_room_message_display_hook` to 'UTF8'. barejid: %s, nick: %s, message: %s", barejid, nick, message);
+        allow_python_threads();
+        return NULL;
+    }
+
     PyObject* p_function;
 
     PyObject* p_module = plugin->module;
