@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -73,14 +73,10 @@ static GList* alert_list;
 void
 cons_debug(const char* const msg, ...)
 {
-    ProfWin* console = wins_get_console();
     if (strcmp(PACKAGE_STATUS, "development") == 0) {
         va_list arg;
         va_start(arg, msg);
-        GString* fmt_msg = g_string_new(NULL);
-        g_string_vprintf(fmt_msg, msg, arg);
-        win_println(console, THEME_DEFAULT, "-", "%s", fmt_msg->str);
-        g_string_free(fmt_msg, TRUE);
+        win_println_va(wins_get_console(), THEME_DEFAULT, "-", msg, arg);
         va_end(arg);
     }
 }
@@ -88,25 +84,20 @@ cons_debug(const char* const msg, ...)
 void
 cons_show(const char* const msg, ...)
 {
-    ProfWin* console = wins_get_console();
     va_list arg;
     va_start(arg, msg);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, msg, arg);
-    win_println(console, THEME_DEFAULT, "-", "%s", fmt_msg->str);
-    g_string_free(fmt_msg, TRUE);
+    win_println_va(wins_get_console(), THEME_DEFAULT, "-", msg, arg);
     va_end(arg);
 }
 
 void
 cons_show_padded(int pad, const char* const msg, ...)
 {
-    ProfWin* console = wins_get_console();
     va_list arg;
     va_start(arg, msg);
     GString* fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, msg, arg);
-    win_println_indent(console, pad, "%s", fmt_msg->str);
+    win_println_indent(wins_get_console(), pad, "%s", fmt_msg->str);
     g_string_free(fmt_msg, TRUE);
     va_end(arg);
 }
@@ -169,12 +160,11 @@ cons_bad_cmd_usage(const char* const cmd)
 void
 cons_show_error(const char* const msg, ...)
 {
-    ProfWin* console = wins_get_console();
     va_list arg;
     va_start(arg, msg);
     GString* fmt_msg = g_string_new(NULL);
     g_string_vprintf(fmt_msg, msg, arg);
-    win_println(console, THEME_ERROR, "-", "%s", fmt_msg->str);
+    win_println(wins_get_console(), THEME_ERROR, "-", "%s", fmt_msg->str);
     g_string_free(fmt_msg, TRUE);
     va_end(arg);
 
@@ -276,20 +266,7 @@ cons_show_tlscert(const TLSCertificate* cert)
 void
 cons_show_typing(const char* const barejid)
 {
-    ProfWin* console = wins_get_console();
-    const char* display_usr = NULL;
-    PContact contact = roster_get_contact(barejid);
-    if (contact) {
-        if (p_contact_name(contact)) {
-            display_usr = p_contact_name(contact);
-        } else {
-            display_usr = barejid;
-        }
-    } else {
-        display_usr = barejid;
-    }
-
-    win_println(console, THEME_TYPING, "-", "!! %s is typing a message…", display_usr);
+    win_println(wins_get_console(), THEME_TYPING, "-", "!! %s is typing a message…", roster_get_display_name(barejid));
     cons_alert(NULL);
 }
 
@@ -414,7 +391,7 @@ cons_about(void)
     }
 
     win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2012 - 2019 James Booth <boothj5web@gmail.com>.");
-    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>.");
+    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>.");
     win_println(console, THEME_DEFAULT, "-", "License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>");
     win_println(console, THEME_DEFAULT, "-", "");
     win_println(console, THEME_DEFAULT, "-", "This is free software; you are free to change and redistribute it.");
@@ -467,8 +444,7 @@ cons_show_login_success(ProfAccount* account, gboolean secured)
 {
     ProfWin* console = wins_get_console();
 
-    const char* fulljid = connection_get_fulljid();
-    win_print(console, THEME_DEFAULT, "-", "%s logged in successfully, ", fulljid);
+    win_print(console, THEME_DEFAULT, "-", "%s logged in successfully, ", connection_get_fulljid());
 
     resource_presence_t presence = accounts_get_login_presence(account->name);
     const char* presence_str = string_from_resource_presence(presence);
@@ -935,17 +911,7 @@ cons_show_status(const char* const barejid)
 void
 cons_show_room_invite(const char* const invitor, const char* const room, const char* const reason)
 {
-    auto_char char* display_from = NULL;
-    PContact contact = roster_get_contact(invitor);
-    if (contact) {
-        if (p_contact_name(contact)) {
-            display_from = strdup(p_contact_name(contact));
-        } else {
-            display_from = strdup(invitor);
-        }
-    } else {
-        display_from = strdup(invitor);
-    }
+    const char* display_from = roster_get_display_name(invitor);
 
     cons_show("");
     cons_show("Chat room invite received:");
@@ -1286,17 +1252,8 @@ cons_titlebar_setting(void)
         cons_show("Titlebar presence (/titlebar)       : OFF");
     }
 
-    if (prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME)) {
-        cons_show("Titlebar show MUC name (/titlebar)       : ON");
-    } else {
-        cons_show("Titlebar show MUC name (/titlebar)       : OFF");
-    }
-
-    if (prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID)) {
-        cons_show("Titlebar show MUC JID (/titlebar)       : ON");
-    } else {
-        cons_show("Titlebar show MUC JID (/titlebar)       : OFF");
-    }
+    auto_gchar gchar* titlebar_muc_title = prefs_get_string(PREF_TITLEBAR_MUC_TITLE);
+    cons_show("MUC window title (/titlebar)        : %s", titlebar_muc_title);
 }
 
 void
@@ -1639,11 +1596,6 @@ cons_roster_setting(void)
         cons_show("Roster unread (/roster)             : OFF");
     }
 
-    if (prefs_get_boolean(PREF_ROSTER_ROOMS))
-        cons_show("Roster rooms (/roster)              : show");
-    else
-        cons_show("Roster rooms (/roster)              : hide");
-
     auto_gchar gchar* priv = prefs_get_string(PREF_ROSTER_PRIVATE);
     if (g_strcmp0(priv, "room") == 0) {
         cons_show("Roster private (/roster)            : room");
@@ -1659,8 +1611,8 @@ cons_roster_setting(void)
     auto_gchar gchar* rooms_by = prefs_get_string(PREF_ROSTER_ROOMS_BY);
     cons_show("Roster rooms by (/roster)           : %s", rooms_by);
 
-    auto_gchar gchar* rooms_use = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
-    cons_show("Roster rooms use (/roster)          : %s", rooms_use);
+    auto_gchar gchar* rooms_title = prefs_get_string(PREF_ROSTER_ROOMS_TITLE);
+    cons_show("Roster rooms title (/roster)        : %s", rooms_title);
 
     auto_gchar gchar* rooms_order = prefs_get_string(PREF_ROSTER_ROOMS_ORDER);
     cons_show("Roster rooms order (/roster)        : %s", rooms_order);
@@ -1958,8 +1910,8 @@ cons_statusbar_setting(void)
     auto_gchar gchar* pref_chat = prefs_get_string(PREF_STATUSBAR_CHAT);
     cons_show("Chat tab display (/statusbar)               : %s", pref_chat);
 
-    auto_gchar gchar* pref_room = prefs_get_string(PREF_STATUSBAR_ROOM);
-    cons_show("Room tab display (/statusbar)               : %s", pref_room);
+    auto_gchar gchar* pref_room_title = prefs_get_string(PREF_STATUSBAR_ROOM_TITLE);
+    cons_show("Room tab display (/statusbar)               : %s", pref_room_title);
 
     auto_gchar gchar* pref_tabmode = prefs_get_string(PREF_STATUSBAR_TABMODE);
     cons_show("Tab mode (/statusbar)                       : %s", pref_tabmode);
@@ -2168,7 +2120,7 @@ cons_correction_setting(void)
         cons_show("Last Message Correction (XEP-0308) (/correction)                   : OFF");
     }
 
-    auto_char char* cc = prefs_get_correction_char();
+    auto_gchar gchar* cc = prefs_get_correction_char();
     cons_show("LMC indication char (/correction char)                             : %s", cc);
 }
 
@@ -2258,7 +2210,7 @@ cons_show_otr_prefs(void)
         cons_show("OTR logging (/otr log)   : Redacted");
     }
 
-    auto_char char* ch = prefs_get_otr_char();
+    auto_gchar gchar* ch = prefs_get_otr_char();
     cons_show("OTR char (/otr char)     : %s", ch);
 
     if (prefs_get_boolean(PREF_OTR_SENDFILE)) {
@@ -2285,7 +2237,7 @@ cons_show_pgp_prefs(void)
         cons_show("PGP logging (/pgp log)   : Redacted");
     }
 
-    auto_char char* ch = prefs_get_pgp_char();
+    auto_gchar gchar* ch = prefs_get_pgp_char();
     cons_show("PGP char (/pgp char)     : %s", ch);
 
     if (prefs_get_boolean(PREF_PGP_SENDFILE)) {
@@ -2315,7 +2267,7 @@ cons_show_omemo_prefs(void)
         cons_show("OMEMO logging (/omemo log)   : Redacted");
     }
 
-    auto_char char* ch = prefs_get_omemo_char();
+    auto_gchar gchar* ch = prefs_get_omemo_char();
     cons_show("OMEMO char (/omemo char)     : %s", ch);
 
     cons_alert(NULL);
@@ -2336,7 +2288,7 @@ cons_show_ox_prefs(void)
         cons_show("OX logging (/ox log)   : Redacted");
     }
 
-    auto_char char* ch = prefs_get_ox_char();
+    auto_gchar gchar* ch = prefs_get_ox_char();
     cons_show("OX char (/ox char)     : %s", ch);
 
     cons_alert(NULL);
@@ -2541,12 +2493,7 @@ cons_alert(ProfWin* alert_origin_window)
     if (current->type != WIN_CONSOLE) {
         status_bar_new(1, WIN_CONSOLE, "console");
 
-        auto_char char* win_name;
-        if (alert_origin_window) {
-            win_name = win_to_string(alert_origin_window);
-        } else {
-            win_name = strdup("console");
-        }
+        auto_gchar gchar* win_name = alert_origin_window ? win_to_string(alert_origin_window) : g_strdup("console");
 
         GList* item = g_list_find_custom(alert_list, win_name, (GCompareFunc)g_strcmp0);
         if (!item) {
@@ -2555,12 +2502,12 @@ cons_alert(ProfWin* alert_origin_window)
     }
 }
 
-char*
+gchar*
 cons_get_string(ProfConsoleWin* conswin)
 {
     assert(conswin != NULL);
 
-    return strdup("Console");
+    return g_strdup("Console");
 }
 
 void
@@ -2881,7 +2828,7 @@ cons_clear_alerts(void)
 void
 cons_remove_alert(ProfWin* window)
 {
-    auto_char char* win_name = win_to_string(window);
+    auto_gchar gchar* win_name = win_to_string(window);
     GList* item = g_list_find_custom(alert_list, win_name, (GCompareFunc)g_strcmp0);
     alert_list = g_list_remove_link(alert_list, item);
     g_list_free_full(item, g_free);
@@ -2937,8 +2884,7 @@ cons_privacy_setting(void)
     }
 
     if (connection_get_status() == JABBER_CONNECTED) {
-        char* account_name = session_get_account_name();
-        ProfAccount* account = accounts_get_account(account_name);
+        ProfAccount* account = accounts_get_account(session_get_account_name());
 
         if (account->client) {
             cons_show("Client name (/account set <account> clientid)              : %s", account->client);
