@@ -136,7 +136,9 @@ static int _inp_rl_subwin_pageup_handler(int count, int key);
 static int _inp_rl_subwin_pagedown_handler(int count, int key);
 static int _inp_rl_startup_hook(void);
 static int _inp_rl_down_arrow_handler(int count, int key);
+static int _inp_rl_scroll_handler(int count, int key);
 static int _inp_rl_send_to_editor(int count, int key);
+static int _inp_rl_print_newline_symbol(int count, int key);
 
 void
 create_input_window(void)
@@ -162,7 +164,6 @@ create_input_window(void)
 
     inp_win = newpad(1, INP_WIN_MAX);
     wbkgd(inp_win, theme_attrs(THEME_INPUT_TEXT));
-    ;
     keypad(inp_win, TRUE);
     wmove(inp_win, 0, 0);
 
@@ -172,8 +173,6 @@ create_input_window(void)
 char*
 inp_readline(void)
 {
-    free(inp_line);
-    inp_line = NULL;
     p_rl_timeout.tv_sec = inp_timeout / 1000;
     p_rl_timeout.tv_usec = inp_timeout % 1000 * 1000;
     FD_ZERO(&fds);
@@ -215,7 +214,9 @@ inp_readline(void)
                 }
             }
         }
-        return strdup(inp_line);
+        char* ret = inp_line;
+        inp_line = NULL;
+        return ret;
     } else {
         return NULL;
     }
@@ -236,15 +237,16 @@ inp_win_resize(void)
     }
 
     wbkgd(inp_win, theme_attrs(THEME_INPUT_TEXT));
-    ;
+
     _inp_win_update_virtual();
 }
 
 void
 inp_nonblocking(gboolean reset)
 {
+    gint inpblock = prefs_get_inpblock();
     if (!prefs_get_boolean(PREF_INPBLOCK_DYNAMIC)) {
-        inp_timeout = prefs_get_inpblock();
+        inp_timeout = inpblock;
         return;
     }
 
@@ -253,14 +255,14 @@ inp_nonblocking(gboolean reset)
         no_input_count = 0;
     }
 
-    if (inp_timeout < prefs_get_inpblock()) {
+    if (inp_timeout < inpblock) {
         no_input_count++;
 
         if (no_input_count % 10 == 0) {
             inp_timeout += no_input_count;
 
-            if (inp_timeout > prefs_get_inpblock()) {
-                inp_timeout = prefs_get_inpblock();
+            if (inp_timeout > inpblock) {
+                inp_timeout = inpblock;
             }
         }
     }
@@ -270,7 +272,10 @@ void
 inp_close(void)
 {
     rl_callback_handler_remove();
+    delwin(inp_win);
+    inp_win = NULL;
     fclose(discard);
+    discard = NULL;
 }
 
 char*
@@ -482,6 +487,7 @@ _inp_rl_addfuncs(void)
     rl_add_funmap_entry("prof_win_close", _inp_rl_win_close_handler);
     rl_add_funmap_entry("prof_send_to_editor", _inp_rl_send_to_editor);
     rl_add_funmap_entry("prof_cut_to_history", _inp_rl_down_arrow_handler);
+    rl_add_funmap_entry("prof_print_newline_symbol", _inp_rl_print_newline_symbol);
 }
 
 // Readline callbacks
@@ -552,9 +558,13 @@ _inp_rl_startup_hook(void)
     rl_bind_key('\t', _inp_rl_tab_handler);
     rl_bind_keyseq("\\e[Z", _inp_rl_shift_tab_handler);
 
+    rl_bind_keyseq("\\e[1;3A", _inp_rl_scroll_handler);     // alt + scroll/arrow up
+    rl_bind_keyseq("\\e[1;3B", _inp_rl_scroll_handler);     // alt + scroll/arrow down
     rl_bind_keyseq("\\e[1;5B", _inp_rl_down_arrow_handler); // ctrl+arrow down
     rl_bind_keyseq("\\eOb", _inp_rl_down_arrow_handler);
 
+    rl_bind_keyseq("\\e\\C-\r", _inp_rl_print_newline_symbol); // alt+enter
+
     // unbind unwanted mappings
     rl_bind_keyseq("\\e=", NULL);
 
@@ -902,7 +912,7 @@ static int
 _inp_rl_win_pageup_handler(int count, int key)
 {
     ProfWin* current = wins_get_current();
-    win_page_up(current);
+    win_page_up(current, 0);
     return 0;
 }
 
@@ -910,7 +920,7 @@ static int
 _inp_rl_win_pagedown_handler(int count, int key)
 {
     ProfWin* current = wins_get_current();
-    win_page_down(current);
+    win_page_down(current, 0);
     return 0;
 }
 
@@ -930,6 +940,21 @@ _inp_rl_subwin_pagedown_handler(int count, int key)
     return 0;
 }
 
+static int
+_inp_rl_scroll_handler(int count, int key)
+{
+    ProfWin* window = wins_get_current();
+
+    if (key == 'B') {
+        // mouse wheel down
+        win_page_down(window, 4);
+    } else if (key == 'A') {
+        // mouse wheel up
+        win_page_up(window, 4);
+    }
+    return 0;
+}
+
 static int
 _inp_rl_down_arrow_handler(int count, int key)
 {
@@ -960,3 +985,10 @@ _inp_rl_send_to_editor(int count, int key)
 
     return 0;
 }
+
+static int
+_inp_rl_print_newline_symbol(int count, int key)
+{
+    rl_insert_text("\n");
+    return 0;
+}
