@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -61,11 +61,11 @@
 #include "ui/screen.h"
 #include "xmpp/xmpp.h"
 #include "xmpp/roster_list.h"
-#include "xmpp/connection.h"
-#include "database.h"
 
-#define CONS_WIN_TITLE "Profanity. Type /help for help information."
-#define XML_WIN_TITLE  "XML Console"
+static const int PAD_SIZE = 100;
+static const char* LOADING_MESSAGE = "Loading older messages…";
+static const char* CONS_WIN_TITLE = "Profanity. Type /help for help information.";
+static const char* XML_WIN_TITLE = "XML Console";
 
 #define CEILING(X) (X - (int)(X) > 0 ? (int)(X + 1) : (int)(X))
 
@@ -133,6 +133,7 @@ win_create_console(void)
 {
     ProfConsoleWin* new_win = malloc(sizeof(ProfConsoleWin));
     new_win->window.type = WIN_CONSOLE;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_split_layout();
 
     return &new_win->window;
@@ -143,6 +144,7 @@ win_create_chat(const char* const barejid)
 {
     ProfChatWin* new_win = malloc(sizeof(ProfChatWin));
     new_win->window.type = WIN_CHAT;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
 
     new_win->barejid = strdup(barejid);
@@ -174,6 +176,7 @@ win_create_muc(const char* const roomjid)
     int cols = getmaxx(stdscr);
 
     new_win->window.type = WIN_MUC;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     ProfLayoutSplit* layout = malloc(sizeof(ProfLayoutSplit));
     layout->base.type = LAYOUT_SPLIT;
 
@@ -228,6 +231,7 @@ win_create_config(const char* const roomjid, DataForm* form, ProfConfWinCallback
 {
     ProfConfWin* new_win = malloc(sizeof(ProfConfWin));
     new_win->window.type = WIN_CONFIG;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
     new_win->roomjid = strdup(roomjid);
     new_win->form = form;
@@ -245,6 +249,7 @@ win_create_private(const char* const fulljid)
 {
     ProfPrivateWin* new_win = malloc(sizeof(ProfPrivateWin));
     new_win->window.type = WIN_PRIVATE;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
     new_win->fulljid = strdup(fulljid);
     new_win->unread = 0;
@@ -261,6 +266,7 @@ win_create_xmlconsole(void)
 {
     ProfXMLWin* new_win = malloc(sizeof(ProfXMLWin));
     new_win->window.type = WIN_XML;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
 
     new_win->memcheck = PROFXMLWIN_MEMCHECK;
@@ -273,6 +279,7 @@ win_create_plugin(const char* const plugin_name, const char* const tag)
 {
     ProfPluginWin* new_win = malloc(sizeof(ProfPluginWin));
     new_win->window.type = WIN_PLUGIN;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
 
     new_win->tag = strdup(tag);
@@ -288,6 +295,7 @@ win_create_vcard(vCard* vcard)
 {
     ProfVcardWin* new_win = malloc(sizeof(ProfVcardWin));
     new_win->window.type = WIN_VCARD;
+    new_win->window.scroll_state = WIN_SCROLL_INNER;
     new_win->window.layout = _win_create_simple_layout();
 
     new_win->vcard = vcard;
@@ -296,90 +304,68 @@ win_create_vcard(vCard* vcard)
     return &new_win->window;
 }
 
-char*
+gchar*
 win_get_title(ProfWin* window)
 {
     if (window == NULL) {
-        return strdup(CONS_WIN_TITLE);
+        return g_strdup(CONS_WIN_TITLE);
     }
 
     switch (window->type) {
     case WIN_CONSOLE:
     {
-        return strdup(CONS_WIN_TITLE);
+        return g_strdup(CONS_WIN_TITLE);
     }
     case WIN_CHAT:
     {
-        ProfChatWin* chatwin = (ProfChatWin*)window;
+        const ProfChatWin* chatwin = (ProfChatWin*)window;
         assert(chatwin->memcheck == PROFCHATWIN_MEMCHECK);
-
-        gboolean show_titlebar_jid = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID);
-        gboolean show_titlebar_name = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME);
-        jabber_conn_status_t conn_status = connection_get_status();
-
-        if (conn_status != JABBER_CONNECTED || !show_titlebar_name) {
-            return strdup(chatwin->barejid);
+        PContact contact = NULL;
+        if (roster_exists()) {
+            contact = roster_get_contact(chatwin->barejid);
         }
-        PContact contact = roster_get_contact(chatwin->barejid);
         if (!contact) {
-            return strdup(chatwin->barejid);
+            return g_strdup(chatwin->barejid);
         }
         const char* name = p_contact_name(contact);
-        if (name == NULL) {
-            return strdup(chatwin->barejid);
+        if (!name) {
+            return g_strdup(chatwin->barejid);
         }
-        if (show_titlebar_jid) {
-            return g_strdup_printf("%s <%s>", name, chatwin->barejid);
-        }
-        return g_strdup_printf("%s", name);
+        return g_strconcat(name, " <", chatwin->barejid, ">", NULL);
     }
     case WIN_MUC:
     {
-        ProfMucWin* mucwin = (ProfMucWin*)window;
+        const ProfMucWin* mucwin = (ProfMucWin*)window;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
-
-        gboolean show_titlebar_jid = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID);
-        gboolean show_titlebar_name = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME);
-        GString* title = g_string_new("");
-
-        if (show_titlebar_name && mucwin->room_name) {
-            g_string_append(title, mucwin->room_name);
-            g_string_append(title, " ");
-        }
-        if (show_titlebar_jid) {
-            g_string_append(title, mucwin->roomjid);
-        }
-
-        return g_string_free(title, FALSE);
+        return mucwin_generate_title(mucwin->roomjid, PREF_TITLEBAR_MUC_TITLE);
     }
     case WIN_CONFIG:
     {
-        if (window->type == WIN_CONFIG) {
-            ProfConfWin* confwin = (ProfConfWin*)window;
-            assert(confwin->memcheck == PROFCONFWIN_MEMCHECK);
-            GString* title = g_string_new(confwin->roomjid);
-            g_string_append(title, " config");
-            if (confwin->form->modified) {
-                g_string_append(title, " *");
-            }
-            return g_string_free(title, FALSE);
+        const ProfConfWin* confwin = (ProfConfWin*)window;
+        assert(confwin->memcheck == PROFCONFWIN_MEMCHECK);
+        auto_gchar gchar* mucwin_title = mucwin_generate_title(confwin->roomjid, PREF_TITLEBAR_MUC_TITLE);
+        if (confwin->form->modified) {
+            return g_strconcat(mucwin_title, " config *", NULL);
         }
+        return g_strconcat(mucwin_title, " config", NULL);
     }
     case WIN_PRIVATE:
     {
-        ProfPrivateWin* privatewin = (ProfPrivateWin*)window;
+        const ProfPrivateWin* privatewin = (ProfPrivateWin*)window;
         assert(privatewin->memcheck == PROFPRIVATEWIN_MEMCHECK);
-        return strdup(privatewin->fulljid);
+        auto_jid Jid* jid = jid_create(privatewin->fulljid);
+        auto_gchar gchar* mucwin_title = mucwin_generate_title(jid->barejid, PREF_TITLEBAR_MUC_TITLE);
+        return g_strconcat(mucwin_title, "/", jid->resourcepart, NULL);
     }
     case WIN_XML:
     {
-        return strdup(XML_WIN_TITLE);
+        return g_strdup(XML_WIN_TITLE);
     }
     case WIN_PLUGIN:
     {
         ProfPluginWin* pluginwin = (ProfPluginWin*)window;
         assert(pluginwin->memcheck == PROFPLUGINWIN_MEMCHECK);
-        return strdup(pluginwin->tag);
+        return g_strdup(pluginwin->tag);
     }
     case WIN_VCARD:
     {
@@ -387,9 +373,8 @@ win_get_title(ProfWin* window)
         assert(vcardwin->memcheck == PROFVCARDWIN_MEMCHECK);
 
         GString* title = g_string_new("vCard ");
-        auto_char char* jid = connection_get_barejid();
 
-        g_string_append(title, jid);
+        g_string_append(title, connection_get_barejid());
 
         if (vcardwin->vcard->modified) {
             g_string_append(title, " *");
@@ -478,7 +463,7 @@ win_get_last_sent_message(ProfWin* window)
     return last_message;
 }
 
-char*
+gchar*
 win_to_string(ProfWin* window)
 {
     assert(window != NULL);
@@ -642,78 +627,110 @@ win_free(ProfWin* window)
 }
 
 void
-win_page_up(ProfWin* window)
+win_page_up(ProfWin* window, int scroll_size)
 {
-    int rows = getmaxy(stdscr);
-    int y = getcury(window->layout->win);
-    int page_space = rows - 4;
     int* page_start = &(window->layout->y_pos);
+    int page_start_initial = *page_start;
+    int total_rows = getcury(window->layout->win);
+    int page_space = getmaxy(stdscr) - 4;
+    if (scroll_size == 0)
+        scroll_size = page_space;
+    win_scroll_state_t* scroll_state = &window->scroll_state;
+    *scroll_state = (*scroll_state == WIN_SCROLL_REACHED_BOTTOM) ? WIN_SCROLL_INNER : *scroll_state;
+    *page_start -= scroll_size;
 
-    *page_start -= page_space;
-
-    if (*page_start == -page_space && window->type == WIN_CHAT) {
+    if (*page_start == -scroll_size && window->type == WIN_CHAT) {
         ProfChatWin* chatwin = (ProfChatWin*)window;
         ProfBuffEntry* first_entry = buffer_size(window->layout->buffer) != 0 ? buffer_get_entry(window->layout->buffer, 0) : NULL;
 
         // Don't do anything if still fetching mam messages
         if (first_entry && !(first_entry->theme_item == THEME_ROOMINFO && g_strcmp0(first_entry->message, LOADING_MESSAGE) == 0)) {
-            if (!chatwin_db_history(chatwin, NULL, NULL, TRUE) && prefs_get_boolean(PREF_MAM)) {
+            if (*scroll_state != WIN_SCROLL_REACHED_TOP) {
+                *scroll_state = !chatwin_db_history(chatwin, NULL, NULL, TRUE) ? WIN_SCROLL_REACHED_TOP : WIN_SCROLL_INNER;
+            }
+
+            if (*scroll_state == WIN_SCROLL_REACHED_TOP && prefs_get_boolean(PREF_MAM)) {
                 win_print_loading_history(window);
                 iq_mam_request_older(chatwin);
             }
+
+            int buff_size = buffer_size(window->layout->buffer);
+            int offset_entry_id = buff_size > 10 ? 10 : buff_size - 1;
+            int offset = buffer_get_entry(window->layout->buffer, offset_entry_id)->y_end_pos;
+            *page_start = offset - page_space;
         }
     }
 
     // went past beginning, show first page
-    if (*page_start < 0)
+    if (*page_start < 0) {
         *page_start = 0;
+    }
 
     window->layout->paged = 1;
-    win_update_virtual(window);
+
+    // update only if position has changed
+    if (page_start_initial != *page_start) {
+        win_update_virtual(window);
+    }
 
     // switch off page if last line and space line visible
-    if ((y) - *page_start == page_space) {
+    if ((total_rows) - *page_start == page_space) {
         window->layout->paged = 0;
     }
 }
 
 void
-win_page_down(ProfWin* window)
+win_page_down(ProfWin* window, int scroll_size)
 {
-    int rows = getmaxy(stdscr);
-    int y = getcury(window->layout->win);
-    int page_space = rows - 4;
+    int total_rows = getcury(window->layout->win);
     int* page_start = &(window->layout->y_pos);
+    int page_space = getmaxy(stdscr) - 4;
+    int page_start_initial = *page_start;
+    if (scroll_size == 0)
+        scroll_size = page_space;
+    win_scroll_state_t* scroll_state = &window->scroll_state;
+    *scroll_state = (*scroll_state == WIN_SCROLL_REACHED_TOP) ? WIN_SCROLL_INNER : *scroll_state;
 
-    *page_start += page_space;
+    *page_start += scroll_size;
 
     // Scrolled down after reaching the bottom of the page
-    if ((*page_start == y || (*page_start == page_space && *page_start >= y)) && window->type == WIN_CHAT) {
+    if ((*page_start > total_rows - page_space || (*page_start == page_space && *page_start >= total_rows)) && window->type == WIN_CHAT) {
         int bf_size = buffer_size(window->layout->buffer);
         if (bf_size > 0) {
-            auto_gchar gchar* start = g_date_time_format_iso8601(buffer_get_entry(window->layout->buffer, bf_size - 1)->time);
+            ProfBuffEntry* last_entry = buffer_get_entry(window->layout->buffer, bf_size - 1);
+            auto_gchar gchar* start = g_date_time_format_iso8601(last_entry->time);
             GDateTime* now = g_date_time_new_now_local();
-            gchar* end = g_date_time_format_iso8601(now);
-            // end is free'd inside
-            chatwin_db_history((ProfChatWin*)window, start, end, FALSE);
+            gchar* end_date = g_date_time_format_iso8601(now);
+            if (*scroll_state != WIN_SCROLL_REACHED_BOTTOM && !chatwin_db_history((ProfChatWin*)window, start, end_date, FALSE)) {
+                *scroll_state = WIN_SCROLL_REACHED_BOTTOM;
+            }
 
             g_date_time_unref(now);
+
+            int offset = last_entry->y_end_pos - 1;
+            *page_start = offset - page_space + scroll_size;
         }
     }
 
-    // only got half a screen, show full screen
-    if ((y - (*page_start)) < page_space)
-        *page_start = y - page_space;
+    total_rows = getcury(window->layout->win);
 
-    // went past end, show full screen
-    else if (*page_start >= y)
-        *page_start = y - page_space - 1;
+    // near the end, but will display only part of the page space, move a bit back to show full page
+    if ((total_rows - *page_start) < page_space) {
+        *page_start = total_rows - page_space;
+        // went past end, show last page
+    } else if (*page_start >= total_rows) {
+        *page_start = total_rows - page_space - 1;
+    }
 
     window->layout->paged = 1;
-    win_update_virtual(window);
+
+    // update only if position has changed
+    if (page_start_initial != *page_start) {
+        win_update_virtual(window);
+    }
 
     // switch off page if last line and space line visible
-    if ((y) - *page_start == page_space) {
+    if (total_rows - *page_start == page_space) {
         window->layout->paged = 0;
     }
 }
@@ -1346,19 +1363,27 @@ win_show_status_string(ProfWin* window, const char* const from,
     win_appendln(window, presence_colour, "");
 }
 
-static void
+/** Corrects the visual representation of a message with prior check for sender validity.
+ *
+ * Returns TRUE if the message was successfully corrected and should not be printed, FALSE otherwise.
+ */
+static gboolean
 _win_correct(ProfWin* window, const char* const message, const char* const id, const char* const replace_id, const char* const from_jid)
 {
+    if (!replace_id) {
+        return FALSE;
+    }
+
     ProfBuffEntry* entry = buffer_get_entry_by_id(window->layout->buffer, replace_id);
     if (!entry) {
-        log_debug("Replace ID %s could not be found in buffer. Message: %s", replace_id, message);
-        return;
+        log_warning("Replace ID %s could not be found in buffer. Message: %s", replace_id, message);
+        return FALSE;
     }
 
     if (g_strcmp0(entry->from_jid, from_jid) != 0) {
         log_debug("Illicit LMC attempt from %s for message from %s with: %s", from_jid, entry->from_jid, message);
         cons_show("Illicit LMC attempt from %s for message from %s", from_jid, entry->from_jid);
-        return;
+        return TRUE;
     }
 
     /*TODO: set date?
@@ -1372,7 +1397,7 @@ _win_correct(ProfWin* window, const char* const message, const char* const id, c
     entry->date = buffer_date_new_now();
     */
 
-    free(entry->show_char);
+    g_free(entry->show_char);
     entry->show_char = prefs_get_correction_char();
 
     if (entry->message) {
@@ -1380,12 +1405,10 @@ _win_correct(ProfWin* window, const char* const message, const char* const id, c
     }
     entry->message = strdup(message);
 
-    if (entry->id) {
-        free(entry->id);
-    }
-    entry->id = strdup(id);
+    // LMC requires original message ID, hence ID remains the same
 
     win_redraw(window);
+    return TRUE;
 }
 
 void
@@ -1400,26 +1423,10 @@ win_print_incoming(ProfWin* window, const char* const display_name_from, ProfMes
     switch (window->type) {
     case WIN_CHAT:
     {
-        auto_char char* enc_char;
         ProfChatWin* chatwin = (ProfChatWin*)window;
+        auto_char char* enc_char = get_enc_char(message->enc, chatwin->incoming_char);
 
-        if (chatwin->incoming_char) {
-            enc_char = strdup(chatwin->incoming_char);
-        } else if (message->enc == PROF_MSG_ENC_OTR) {
-            enc_char = prefs_get_otr_char();
-        } else if (message->enc == PROF_MSG_ENC_PGP) {
-            enc_char = prefs_get_pgp_char();
-        } else if (message->enc == PROF_MSG_ENC_OX) { // XEP-0373: OpenPGP for XMPP
-            enc_char = prefs_get_ox_char();
-        } else if (message->enc == PROF_MSG_ENC_OMEMO) {
-            enc_char = prefs_get_omemo_char();
-        } else {
-            enc_char = strdup("-");
-        }
-
-        if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && message->replace_id) {
-            _win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->barejid);
-        } else {
+        if (!prefs_get_boolean(PREF_CORRECTION_ALLOW) || !_win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->barejid)) {
             // Prevent duplicate messages when current client is sending a message or if it's mam
             if (g_strcmp0(message->from_jid->fulljid, connection_get_fulljid()) != 0 && !message->is_mam) {
                 _win_printf(window, enc_char, 0, message->timestamp, flags, THEME_TEXT_THEM, display_name_from, message->from_jid->barejid, message->id, "%s", message->plain);
@@ -1445,9 +1452,7 @@ win_print_them(ProfWin* window, theme_item_t theme_item, const char* const show_
 void
 win_println_incoming_muc_msg(ProfWin* window, char* show_char, int flags, const ProfMessage* const message)
 {
-    if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && message->replace_id) {
-        _win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->fulljid);
-    } else {
+    if (!prefs_get_boolean(PREF_CORRECTION_ALLOW) || !_win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->fulljid)) {
         _win_printf(window, show_char, 0, message->timestamp, flags | NO_ME, THEME_TEXT_THEM, message->from_jid->resourcepart, message->from_jid->fulljid, message->id, "%s", message->plain);
     }
 
@@ -1459,9 +1464,7 @@ win_print_outgoing_muc_msg(ProfWin* window, char* show_char, const char* const m
 {
     GDateTime* timestamp = g_date_time_new_now_local();
 
-    if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && replace_id) {
-        _win_correct(window, message, id, replace_id, me);
-    } else {
+    if (!prefs_get_boolean(PREF_CORRECTION_ALLOW) || !_win_correct(window, message, id, replace_id, me)) {
         _win_printf(window, show_char, 0, timestamp, 0, THEME_TEXT_ME, me, me, id, "%s", message);
     }
 
@@ -1475,9 +1478,7 @@ win_print_outgoing(ProfWin* window, const char* show_char, const char* const id,
     GDateTime* timestamp = g_date_time_new_now_local();
 
     const char* myjid = connection_get_fulljid();
-    if (replace_id) {
-        _win_correct(window, message, id, replace_id, myjid);
-    } else {
+    if (!_win_correct(window, message, id, replace_id, myjid)) {
         auto_gchar gchar* outgoing_str = prefs_get_string(PREF_OUTGOING_STAMP);
         _win_printf(window, show_char, 0, timestamp, 0, THEME_TEXT_ME, outgoing_str, myjid, id, "%s", message);
     }
@@ -1491,22 +1492,15 @@ win_print_history(ProfWin* window, const ProfMessage* const message)
 {
     g_date_time_ref(message->timestamp);
 
-    auto_gchar gchar* display_name;
     int flags = 0;
-    const char* jid = connection_get_fulljid();
-    auto_jid Jid* jidp = jid_create(jid);
+    auto_gchar gchar* display_name = get_display_name(message, &flags);
+    auto_char char* ch = get_show_char(message->enc);
 
-    if (g_strcmp0(jidp->barejid, message->from_jid->barejid) == 0) {
-        display_name = strdup("me");
-    } else {
-        display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
-        flags = NO_ME;
-    }
-
-    buffer_append(window->layout->buffer, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, NULL, message->plain, NULL, NULL);
     wins_add_urls_ac(window, message, FALSE);
     wins_add_quotes_ac(window, message->plain, FALSE);
-    _win_print_internal(window, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
+    int y_start_pos = getcury(window->layout->win);
+    _win_print_internal(window, ch, 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
+    buffer_append(window->layout->buffer, ch, 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->from_jid->barejid, message->plain, NULL, message->id, y_start_pos, getcury(window->layout->win));
 
     inp_nonblocking(TRUE);
     g_date_time_unref(message->timestamp);
@@ -1517,164 +1511,102 @@ win_print_old_history(ProfWin* window, const ProfMessage* const message)
 {
     g_date_time_ref(message->timestamp);
 
-    auto_char char* display_name;
     int flags = 0;
-    const char* jid = connection_get_fulljid();
-    auto_jid Jid* jidp = jid_create(jid);
+    auto_gchar gchar* display_name = get_display_name(message, &flags);
 
-    if (g_strcmp0(jidp->barejid, message->from_jid->barejid) == 0) {
-        display_name = strdup("me");
-    } else {
-        display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
-        flags = NO_ME;
-    }
+    auto_char char* ch = get_show_char(message->enc);
 
-    buffer_prepend(window->layout->buffer, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, NULL, message->plain, NULL, NULL);
+    int y_start_pos = getcury(window->layout->win);
     wins_add_urls_ac(window, message, TRUE);
     wins_add_quotes_ac(window, message->plain, TRUE);
-    _win_print_internal(window, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
+    _win_print_internal(window, ch, 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
+    buffer_prepend(window->layout->buffer, ch, 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->from_jid->barejid, message->plain, NULL, message->id, y_start_pos, getcury(window->layout->win));
 
     inp_nonblocking(TRUE);
     g_date_time_unref(message->timestamp);
 }
 
-void
-win_print(ProfWin* window, theme_item_t theme_item, const char* show_char, const char* const message, ...)
+static void
+win_println_va_internal(ProfWin* window, theme_item_t theme_item, int pad, int flags, const char* show_char, const char* const message, va_list arg)
 {
     GDateTime* timestamp = g_date_time_new_now_local();
 
-    va_list arg;
-    va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
+    auto_gchar gchar* msg = g_strdup_vprintf(message, arg);
 
-    buffer_append(window->layout->buffer, show_char, 0, timestamp, NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, show_char, 0, timestamp, NO_EOL, theme_item, "", fmt_msg->str, NULL);
+    int y_start_pos = getcury(window->layout->win);
+    _win_print_internal(window, show_char, pad, timestamp, flags, theme_item, "", msg, NULL);
+    buffer_append(window->layout->buffer, show_char, pad, timestamp, flags, theme_item, "", NULL, msg, NULL, NULL, y_start_pos, getcury(window->layout->win));
 
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
+}
 
-    g_string_free(fmt_msg, TRUE);
+void
+win_print(ProfWin* window, theme_item_t theme_item, const char* show_char, const char* const message, ...)
+{
+    va_list arg;
+    va_start(arg, message);
+    win_println_va_internal(window, theme_item, 0, NO_EOL, show_char, message, arg);
     va_end(arg);
 }
 
+void
+win_println_va(ProfWin* window, theme_item_t theme_item, const char* show_char, const char* const message, va_list arg)
+{
+    win_println_va_internal(window, theme_item, 0, 0, show_char, message, arg);
+}
+
 void
 win_println(ProfWin* window, theme_item_t theme_item, const char* show_char, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, show_char, 0, timestamp, 0, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, show_char, 0, timestamp, 0, theme_item, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, theme_item, 0, 0, show_char, message, arg);
     va_end(arg);
 }
 
 void
 win_println_indent(ProfWin* window, int pad, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, "-", pad, timestamp, 0, THEME_DEFAULT, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, "-", pad, timestamp, 0, THEME_DEFAULT, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, THEME_DEFAULT, pad, 0, "-", message, arg);
     va_end(arg);
 }
 
 void
 win_append(ProfWin* window, theme_item_t theme_item, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_EOL, theme_item, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, theme_item, 0, NO_DATE | NO_EOL, "-", message, arg);
     va_end(arg);
 }
 
 void
 win_appendln(ProfWin* window, theme_item_t theme_item, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, "-", 0, timestamp, NO_DATE, theme_item, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, theme_item, 0, NO_DATE, "-", message, arg);
     va_end(arg);
 }
 
 void
 win_append_highlight(ProfWin* window, theme_item_t theme_item, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_ME | NO_EOL, theme_item, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, theme_item, 0, NO_DATE | NO_ME | NO_EOL, "-", message, arg);
     va_end(arg);
 }
 
 void
 win_appendln_highlight(ProfWin* window, theme_item_t theme_item, const char* const message, ...)
 {
-    GDateTime* timestamp = g_date_time_new_now_local();
-
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
-
-    buffer_append(window->layout->buffer, "-", 0, timestamp, NO_DATE | NO_ME, theme_item, "", NULL, fmt_msg->str, NULL, NULL);
-    _win_print_internal(window, "-", 0, timestamp, NO_DATE | NO_ME, theme_item, "", fmt_msg->str, NULL);
-
-    inp_nonblocking(TRUE);
-    g_date_time_unref(timestamp);
-
-    g_string_free(fmt_msg, TRUE);
+    win_println_va_internal(window, theme_item, 0, NO_DATE | NO_ME, "-", message, arg);
     va_end(arg);
 }
 
@@ -1685,7 +1617,7 @@ win_print_http_transfer(ProfWin* window, const char* const message, char* id)
 }
 
 void
-win_print_outgoing_with_receipt(ProfWin* window, const char* show_char, const char* const from, const char* const message, char* id, const char* const replace_id)
+win_print_outgoing_with_receipt(ProfWin* window, const char* show_char, const char* const from, const char* const message, const char* id, const char* const replace_id)
 {
     GDateTime* time = g_date_time_new_now_local();
 
@@ -1693,12 +1625,12 @@ win_print_outgoing_with_receipt(ProfWin* window, const char* show_char, const ch
     receipt->received = FALSE;
 
     const char* myjid = connection_get_fulljid();
-    if (replace_id) {
-        _win_correct(window, message, id, replace_id, myjid);
+    if (_win_correct(window, message, id, replace_id, myjid)) {
         free(receipt); // TODO: probably we should use this in _win_correct()
     } else {
-        buffer_append(window->layout->buffer, show_char, 0, time, 0, THEME_TEXT_ME, from, myjid, message, receipt, id);
+        int y_start_pos = getcury(window->layout->win);
         _win_print_internal(window, show_char, 0, time, 0, THEME_TEXT_ME, from, message, receipt);
+        buffer_append(window->layout->buffer, show_char, 0, time, 0, THEME_TEXT_ME, from, myjid, message, receipt, id, y_start_pos, getcury(window->layout->win));
     }
 
     // TODO: cross-reference.. this should be replaced by a real event-based system
@@ -1754,16 +1686,16 @@ _win_printf(ProfWin* window, const char* show_char, int pad_indent, GDateTime* t
 
     va_list arg;
     va_start(arg, message);
-    GString* fmt_msg = g_string_new(NULL);
-    g_string_vprintf(fmt_msg, message, arg);
 
-    buffer_append(window->layout->buffer, show_char, pad_indent, timestamp, flags, theme_item, display_from, from_jid, fmt_msg->str, NULL, message_id);
-    _win_print_internal(window, show_char, pad_indent, timestamp, flags, theme_item, display_from, fmt_msg->str, NULL);
+    auto_gchar gchar* msg = g_strdup_vprintf(message, arg);
+
+    int y_start_pos = getcury(window->layout->win);
+    _win_print_internal(window, show_char, pad_indent, timestamp, flags, theme_item, display_from, msg, NULL);
+    buffer_append(window->layout->buffer, show_char, pad_indent, timestamp, flags, theme_item, display_from, from_jid, msg, NULL, message_id, y_start_pos, getcury(window->layout->win));
 
     inp_nonblocking(TRUE);
     g_date_time_unref(timestamp);
 
-    g_string_free(fmt_msg, TRUE);
     va_end(arg);
 }
 
@@ -2038,13 +1970,13 @@ win_print_trackbar(ProfWin* window)
 void
 win_redraw(ProfWin* window)
 {
-    int size;
+    int size = buffer_size(window->layout->buffer);
     werase(window->layout->win);
-    size = buffer_size(window->layout->buffer);
 
     for (int i = 0; i < size; i++) {
         ProfBuffEntry* e = buffer_get_entry(window->layout->buffer, i);
 
+        e->y_start_pos = getcury(window->layout->win);
         if (e->display_from == NULL && e->message && e->message[0] == '-') {
             // just an indicator to print the trackbar/separator not the actual message
             win_print_trackbar(window);
@@ -2052,6 +1984,7 @@ win_redraw(ProfWin* window)
             // regular thing to print
             _win_print_internal(window, e->show_char, e->pad_indent, e->time, e->flags, e->theme_item, e->display_from, e->message, e->receipt);
         }
+        e->y_end_pos = getcury(window->layout->win);
     }
 }
 
@@ -2067,7 +2000,8 @@ win_print_loading_history(ProfWin* window)
         timestamp = g_date_time_new_now_local();
     }
 
-    buffer_prepend(window->layout->buffer, "-", 0, timestamp, NO_DATE, THEME_ROOMINFO, NULL, NULL, LOADING_MESSAGE, NULL, NULL);
+    int cur_y = getcury(window->layout->win);
+    buffer_prepend(window->layout->buffer, "-", 0, timestamp, NO_DATE, THEME_ROOMINFO, NULL, NULL, LOADING_MESSAGE, NULL, NULL, cur_y, cur_y + 1);
 
     if (is_buffer_empty)
         g_date_time_unref(timestamp);
@@ -2223,12 +2157,10 @@ win_command_exec_error(ProfWin* window, const char* const command, const char* c
     assert(window != NULL);
     va_list arg;
     va_start(arg, error);
-    GString* msg = g_string_new(NULL);
-    g_string_vprintf(msg, error, arg);
+    auto_gchar gchar* msg = g_strdup_vprintf(error, arg);
 
-    win_println(window, THEME_ERROR, "!", "Error executing command %s: %s", command, msg->str);
+    win_println(window, THEME_ERROR, "!", "Error executing command %s: %s", command, msg);
 
-    g_string_free(msg, TRUE);
     va_end(arg);
 }
 
@@ -2287,7 +2219,8 @@ win_insert_last_read_position_marker(ProfWin* window, char* id)
     // the trackbar/separator will actually be print in win_redraw().
     // this only puts it in the buffer and win_redraw() will interpret it.
     // so that we have the correct length even when resizing.
-    buffer_append(window->layout->buffer, " ", 0, time, 0, THEME_TEXT, NULL, NULL, "-", NULL, id);
+    int y_start_pos = getcury(window->layout->win);
+    buffer_append(window->layout->buffer, " ", 0, time, 0, THEME_TEXT, NULL, NULL, "-", NULL, id, y_start_pos, getcury(window->layout->win));
     win_redraw(window);
 
     g_date_time_unref(time);
@@ -2308,9 +2241,35 @@ win_quote_autocomplete(ProfWin* window, const char* const input, gboolean previo
     auto_gcharv gchar** parts = g_strsplit(result, "\n", -1);
     auto_gchar gchar* quoted_result = g_strjoinv("\n> ", parts);
 
-    GString* replace_with = g_string_new("> ");
-    g_string_append(replace_with, quoted_result);
-    g_string_append(replace_with, "\n");
+    return g_strdup_printf("> %s\n", quoted_result);
+}
 
-    return g_string_free(replace_with, FALSE);
+// Derive encryption char from encryption mode. Output needs to be freed by caller.
+char*
+get_show_char(prof_enc_t encryption_mode)
+{
+    char* enc_char;
+
+    if (encryption_mode == PROF_MSG_ENC_OTR) {
+        enc_char = prefs_get_otr_char();
+    } else if (encryption_mode == PROF_MSG_ENC_PGP) {
+        enc_char = prefs_get_pgp_char();
+    } else if (encryption_mode == PROF_MSG_ENC_OMEMO) {
+        enc_char = prefs_get_omemo_char();
+    } else if (encryption_mode == PROF_MSG_ENC_OX) {
+        enc_char = prefs_get_ox_char();
+    } else {
+        enc_char = strdup("-");
+    }
+
+    return enc_char;
+}
+
+char*
+get_enc_char(prof_enc_t enc_mode, const char* alt)
+{
+    if (alt)
+        return strdup(alt);
+    else
+        return get_show_char(enc_mode);
 }
