@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -1134,7 +1134,7 @@ wins_create_summary(gboolean unread)
             GString* line = g_string_new("");
 
             int ui_index = GPOINTER_TO_INT(curr->data);
-            auto_char char* winstring = win_to_string(window);
+            auto_gchar gchar* winstring = win_to_string(window);
             if (!winstring) {
                 g_string_free(line, TRUE);
                 continue;
@@ -1179,7 +1179,7 @@ wins_create_summary_attention()
             GString* line = g_string_new("");
 
             int ui_index = GPOINTER_TO_INT(curr->data);
-            auto_char char* winstring = win_to_string(window);
+            auto_gchar gchar* winstring = win_to_string(window);
             if (!winstring) {
                 g_string_free(line, TRUE);
                 continue;
@@ -1226,8 +1226,11 @@ void
 wins_destroy(void)
 {
     g_hash_table_destroy(windows);
+    windows = NULL;
     autocomplete_free(wins_ac);
+    wins_ac = NULL;
     autocomplete_free(wins_close_ac);
+    wins_close_ac = NULL;
 }
 
 ProfWin*
@@ -1310,7 +1313,8 @@ wins_add_urls_ac(const ProfWin* const win, const ProfMessage* const message, con
     GRegex* regex;
     GMatchInfo* match_info;
 
-    regex = g_regex_new("(https?|aesgcm)://\\S+", 0, 0, NULL);
+    // https://stackoverflow.com/questions/43588699/regex-for-matching-any-url-character
+    regex = g_regex_new("(https?|aesgcm)://[\\w\\-.~:/?#\\[\\]@!$&'()*+,;=%]+", 0, 0, NULL);
     g_regex_match(regex, message->plain, 0, &match_info);
 
     while (g_match_info_matches(match_info)) {
