@@ -35,10 +35,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_GIT_VERSION
-#include "gitversion.h"
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -58,32 +54,44 @@
 #include "xmpp/form.h"
 #include "xmpp/capabilities.h"
 
-static char* cache_loc;
+static gchar* cache_loc;
+static prof_keyfile_t caps_prof_keyfile;
 static GKeyFile* cache;
 
 static GHashTable* jid_to_ver;
 static GHashTable* jid_to_caps;
 
 static GHashTable* prof_features;
-static char* my_sha1;
+static gchar* my_sha1;
 
 static void _save_cache(void);
 static EntityCapabilities* _caps_by_ver(const char* const ver);
 static EntityCapabilities* _caps_by_jid(const char* const jid);
 static EntityCapabilities* _caps_copy(EntityCapabilities* caps);
 
+static void
+_caps_close(void)
+{
+    caps_reset_ver();
+    free_keyfile(&caps_prof_keyfile);
+    cache = NULL;
+    g_hash_table_destroy(jid_to_ver);
+    g_hash_table_destroy(jid_to_caps);
+    g_free(cache_loc);
+    cache_loc = NULL;
+    g_hash_table_destroy(prof_features);
+    prof_features = NULL;
+}
+
 void
 caps_init(void)
 {
     log_info("Loading capabilities cache");
-    cache_loc = files_get_data_path(FILE_CAPSCACHE);
 
-    if (g_file_test(cache_loc, G_FILE_TEST_EXISTS)) {
-        g_chmod(cache_loc, S_IRUSR | S_IWUSR);
-    }
+    prof_add_shutdown_routine(_caps_close);
 
-    cache = g_key_file_new();
-    g_key_file_load_from_file(cache, cache_loc, G_KEY_FILE_KEEP_COMMENTS, NULL);
+    load_data_keyfile(&caps_prof_keyfile, FILE_CAPSCACHE);
+    cache = caps_prof_keyfile.keyfile;
 
     jid_to_ver = g_hash_table_new_full(g_str_hash, g_str_equal, free, free);
     jid_to_caps = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)caps_destroy);
@@ -348,19 +356,6 @@ caps_reset_ver(void)
     }
 }
 
-void
-caps_close(void)
-{
-    g_key_file_free(cache);
-    cache = NULL;
-    g_hash_table_destroy(jid_to_ver);
-    g_hash_table_destroy(jid_to_caps);
-    free(cache_loc);
-    cache_loc = NULL;
-    g_hash_table_destroy(prof_features);
-    prof_features = NULL;
-}
-
 static EntityCapabilities*
 _caps_by_ver(const char* const ver)
 {
@@ -368,17 +363,17 @@ _caps_by_ver(const char* const ver)
         return NULL;
     }
 
-    char* category = g_key_file_get_string(cache, ver, "category", NULL);
-    char* type = g_key_file_get_string(cache, ver, "type", NULL);
-    char* name = g_key_file_get_string(cache, ver, "name", NULL);
+    auto_gchar gchar* category = g_key_file_get_string(cache, ver, "category", NULL);
+    auto_gchar gchar* type = g_key_file_get_string(cache, ver, "type", NULL);
+    auto_gchar gchar* name = g_key_file_get_string(cache, ver, "name", NULL);
 
-    char* software = g_key_file_get_string(cache, ver, "software", NULL);
-    char* software_version = g_key_file_get_string(cache, ver, "software_version", NULL);
-    char* os = g_key_file_get_string(cache, ver, "os", NULL);
-    char* os_version = g_key_file_get_string(cache, ver, "os_version", NULL);
+    auto_gchar gchar* software = g_key_file_get_string(cache, ver, "software", NULL);
+    auto_gchar gchar* software_version = g_key_file_get_string(cache, ver, "software_version", NULL);
+    auto_gchar gchar* os = g_key_file_get_string(cache, ver, "os", NULL);
+    auto_gchar gchar* os_version = g_key_file_get_string(cache, ver, "os_version", NULL);
 
     gsize features_len = 0;
-    gchar** features_list = g_key_file_get_string_list(cache, ver, "features", &features_len, NULL);
+    auto_gcharv gchar** features_list = g_key_file_get_string_list(cache, ver, "features", &features_len, NULL);
     GSList* features = NULL;
     if (features_list && features_len > 0) {
         for (int i = 0; i < features_len; i++) {
@@ -391,16 +386,6 @@ _caps_by_ver(const char* const ver)
         software, software_version, os, os_version,
         features);
 
-    g_free(category);
-    g_free(type);
-    g_free(name);
-    g_free(software);
-    g_free(software_version);
-    g_free(os);
-    g_free(os_version);
-    if (features_list) {
-        g_strfreev(features_list);
-    }
     g_slist_free(features);
 
     return result;
@@ -470,9 +455,5 @@ caps_destroy(EntityCapabilities* caps)
 static void
 _save_cache(void)
 {
-    gsize g_data_size;
-    gchar* g_cache_data = g_key_file_to_data(cache, &g_data_size, NULL);
-    g_file_set_contents(cache_loc, g_cache_data, g_data_size, NULL);
-    g_chmod(cache_loc, S_IRUSR | S_IWUSR);
-    g_free(g_cache_data);
+    save_keyfile(&caps_prof_keyfile);
 }
