@@ -104,6 +104,17 @@ roster_create(void)
     roster_pending_presence = NULL;
 }
 
+static void
+_pendingPresence_free(ProfPendingPresence* presence)
+{
+    if (!presence)
+        return;
+    if (presence->last_activity)
+        g_date_time_unref(presence->last_activity);
+    free(presence->barejid);
+    free(presence);
+}
+
 void
 roster_destroy(void)
 {
@@ -119,6 +130,10 @@ roster_destroy(void)
 
     free(roster);
     roster = NULL;
+
+    if (roster_pending_presence)
+        g_slist_free_full(roster_pending_presence, (GDestroyNotify)_pendingPresence_free);
+    roster_pending_presence = NULL;
 }
 
 gboolean
@@ -168,25 +183,12 @@ roster_get_contact(const char* const barejid)
     return contact;
 }
 
-char*
+const char*
 roster_get_display_name(const char* const barejid)
 {
     assert(roster != NULL);
 
-    GString* result = g_string_new("");
-
-    PContact contact = roster_get_contact(barejid);
-    if (contact) {
-        if (p_contact_name(contact)) {
-            g_string_append(result, p_contact_name(contact));
-        } else {
-            g_string_append(result, barejid);
-        }
-    } else {
-        g_string_append(result, barejid);
-    }
-
-    return g_string_free(result, FALSE);
+    return p_contact_name(roster_get_contact(barejid)) ?: barejid;
 }
 
 gchar*
@@ -200,25 +202,12 @@ roster_get_msg_display_name(const char* const barejid, const char* const resourc
         return incoming_str;
     }
 
-    GString* result = g_string_new("");
-
-    PContact contact = roster_get_contact(barejid);
-    if (contact) {
-        if (p_contact_name(contact)) {
-            g_string_append(result, p_contact_name(contact));
-        } else {
-            g_string_append(result, barejid);
-        }
-    } else {
-        g_string_append(result, barejid);
-    }
+    const char* contact_name = roster_get_display_name(barejid);
 
     if (resource && prefs_get_boolean(PREF_RESOURCE_MESSAGE)) {
-        g_string_append(result, "/");
-        g_string_append(result, resource);
+        return g_strdup_printf("%s/%s", contact_name, resource);
     }
-
-    return g_string_free(result, FALSE);
+    return g_strdup(contact_name);
 }
 
 gboolean
@@ -716,15 +705,6 @@ roster_compare_presence(PContact a, PContact b)
     }
 }
 
-static void
-_pendingPresence_free(ProfPendingPresence* presence)
-{
-    if (!presence)
-        return;
-    free(presence->barejid);
-    free(presence);
-}
-
 void
 roster_process_pending_presence(void)
 {
@@ -734,10 +714,6 @@ roster_process_pending_presence(void)
     for (iter = roster_pending_presence; iter != NULL; iter = iter->next) {
         ProfPendingPresence* presence = iter->data;
         roster_update_presence(presence->barejid, presence->resource, presence->last_activity);
-        /* seems like resource isn't free on the calling side */
-        if (presence->last_activity) {
-            g_date_time_unref(presence->last_activity);
-        }
     }
 
     g_slist_free_full(roster_pending_presence, (GDestroyNotify)_pendingPresence_free);
