@@ -2,7 +2,7 @@
  * xmpp.h
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2025 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -64,6 +64,7 @@
 #define XMPP_FEATURE_USER_AVATAR_METADATA_NOTIFY "urn:xmpp:avatar:metadata+notify"
 #define XMPP_FEATURE_LAST_MESSAGE_CORRECTION     "urn:xmpp:message-correct:0"
 #define XMPP_FEATURE_MAM2                        "urn:xmpp:mam:2"
+#define XMPP_FEATURE_MAM2_EXTENDED               "urn:xmpp:mam:2#extended"
 #define XMPP_FEATURE_SPAM_REPORTING              "urn:xmpp:reporting:1"
 
 typedef enum {
@@ -182,24 +183,28 @@ jabber_conn_status_t session_connect_with_details(const char* const jid, const c
 jabber_conn_status_t session_connect_with_account(const ProfAccount* const account);
 
 void session_disconnect(void);
-void session_shutdown(void);
 void session_process_events(void);
-char* session_get_account_name(void);
+const char* session_get_account_name(void);
+void session_reconnect_now(void);
 
+void connection_disconnect(void);
 jabber_conn_status_t connection_get_status(void);
-char* connection_get_presence_msg(void);
+const char* connection_get_presence_msg(void);
 void connection_set_presence_msg(const char* const message);
 const char* connection_get_fulljid(void);
-char* connection_get_barejid(void);
-char* connection_get_user(void);
+const Jid* connection_get_jid(void);
+const char* connection_get_barejid(void);
+gboolean equals_our_barejid(const char* cmp);
+const char* connection_get_user(void);
 char* connection_create_uuid(void);
 void connection_free_uuid(char* uuid);
 TLSCertificate* connection_get_tls_peer_cert(void);
 gboolean connection_is_secured(void);
 gboolean connection_send_stanza(const char* const stanza);
 GList* connection_get_available_resources(void);
+int connection_count_available_resources(void);
 gboolean connection_supports(const char* const feature);
-char* connection_jid_for_feature(const char* const feature);
+const char* connection_jid_for_feature(const char* const feature);
 
 const char* connection_get_profanity_identifier(void);
 
@@ -236,11 +241,12 @@ void iq_enable_carbons(void);
 void iq_disable_carbons(void);
 void iq_send_software_version(const char* const fulljid);
 void iq_rooms_cache_clear(void);
-void iq_handlers_clear();
-void iq_room_list_request(gchar* conferencejid, gchar* filter);
-void iq_disco_info_request(gchar* jid);
-void iq_disco_items_request(gchar* jid);
-void iq_last_activity_request(gchar* jid);
+void iq_handlers_remove_win(ProfWin* window);
+void iq_handlers_clear(void);
+void iq_room_list_request(const char* conferencejid, char* filter);
+void iq_disco_info_request(const char* jid);
+void iq_disco_items_request(const char* jid);
+void iq_last_activity_request(const char* jid);
 void iq_set_autoping(int seconds);
 void iq_confirm_instant_room(const char* const room_jid);
 void iq_destroy_room(const char* const room_jid);
@@ -260,12 +266,14 @@ void iq_autoping_check(void);
 void iq_http_upload_request(HTTPUpload* upload);
 void iq_command_list(const char* const target);
 void iq_command_exec(const char* const target, const char* const command);
-void iq_mam_request(ProfChatWin* win);
+void iq_mam_request(ProfChatWin* win, GDateTime* enddate);
+void iq_mam_request_older(ProfChatWin* win);
 void iq_register_change_password(const char* const user, const char* const password);
 void iq_muc_register_nick(const char* const roomjid);
 
+void autoping_timer_extend(void);
+
 EntityCapabilities* caps_lookup(const char* const jid);
-void caps_close(void);
 void caps_destroy(EntityCapabilities* caps);
 void caps_reset_ver(void);
 void caps_add_feature(char* feature);
@@ -310,4 +318,5 @@ void form_reset_autocompleters(DataForm* form);
 
 void publish_user_mood(const char* const mood, const char* const text);
 
+gchar* get_display_name(const ProfMessage* const message, int* flags);
 #endif
