@@ -4,8 +4,8 @@
 #define XDG_CONFIG_HOME "./tests/functionaltests/files/xdg_config_home"
 #define XDG_DATA_HOME   "./tests/functionaltests/files/xdg_data_home"
 
-void init_prof_test(void **state);
-void close_prof_test(void **state);
+int init_prof_test(void **state);
+int close_prof_test(void **state);
 
 void prof_start(void);
 void prof_connect(void);
