@@ -10,19 +10,15 @@ void
 accounts_load(void)
 {
 }
-void
-accounts_close(void)
-{
-}
 
 char*
-accounts_find_all(char* prefix)
+accounts_find_all(const char* const prefix, gboolean previous, void* context)
 {
     return NULL;
 }
 
 char*
-accounts_find_enabled(char* prefix, void* context)
+accounts_find_enabled(const char* const prefix, gboolean previous, void* context)
 {
     return NULL;
 }
@@ -37,7 +33,7 @@ accounts_reset_enabled_search(void)
 }
 
 void
-accounts_add(const char* jid, const char* altdomain, const int port)
+accounts_add(const char* jid, const char* altdomain, const int port, const char* const tls_policy, const char* const auth_policy)
 {
     check_expected(jid);
     check_expected(altdomain);
