@@ -3,8 +3,9 @@
 #include <stdlib.h>
 
 void avatar_pep_subscribe(void){};
+
 gboolean
-avatar_get_by_nick(const char* nick)
+avatar_get_by_nick(const char* nick, gboolean open)
 {
     return TRUE;
 }
