@@ -47,10 +47,6 @@ session_disconnect(void)
 {
 }
 void
-session_shutdown(void)
-{
-}
-void
 session_process_events(void)
 {
 }
@@ -58,22 +54,33 @@ void
 connection_disconnect(void)
 {
 }
+const Jid*
+connection_get_jid(void)
+{
+    return mock_ptr_type(Jid*);
+}
 const char*
 connection_get_fulljid(void)
 {
     return mock_ptr_type(char*);
 }
 
-char*
+const char*
 connection_get_barejid(void)
 {
-    return mock_ptr_type(char*);
+    return mock_ptr_type(const char*);
 }
 
-char*
+gboolean
+equals_our_barejid(const char* cmp)
+{
+    return TRUE;
+}
+
+const char*
 connection_get_user(void)
 {
-    return mock_ptr_type(char*);
+    return mock_ptr_type(const char*);
 }
 
 const char*
@@ -111,13 +118,13 @@ connection_get_status(void)
     return mock_type(jabber_conn_status_t);
 }
 
-char*
+const char*
 connection_get_presence_msg(void)
 {
-    return mock_ptr_type(char*);
+    return mock_ptr_type(const char*);
 }
 
-char*
+const char*
 session_get_account_name(void)
 {
     return mock_ptr_type(char*);
@@ -313,18 +320,18 @@ iq_send_software_version(const char* const fulljid)
 }
 
 void
-iq_room_list_request(gchar* conferencejid, gchar* filter)
+iq_room_list_request(const char* conferencejid, char* filter)
 {
     check_expected(conferencejid);
     check_expected(filter);
 }
 
 void
-iq_disco_info_request(gchar* jid)
+iq_disco_info_request(const char* jid)
 {
 }
 void
-iq_disco_items_request(gchar* jid)
+iq_disco_items_request(const char* jid)
 {
 }
 void
@@ -401,7 +408,7 @@ iq_room_role_list(const char* const room, char* role)
 {
 }
 void
-iq_last_activity_request(gchar* jid)
+iq_last_activity_request(const char* jid)
 {
 }
 void
@@ -439,6 +446,11 @@ iq_mam_request(ProfChatWin* win, GDateTime* enddate)
 {
 }
 
+void
+iq_feature_retrieval_complete_handler(void)
+{
+}
+
 void
 publish_user_mood(const char* const mood, const char* const text)
 {
@@ -461,11 +473,6 @@ caps_lookup(const char* const jid)
     return NULL;
 }
 
-void
-caps_close(void)
-{
-}
-
 void
 caps_destroy(EntityCapabilities* caps)
 {
